
  IMPLEMENTATION MODULE Bits;

  (*******************************************************************

    Module  Bits     (Version 1.0)

              Swiss Federal Institute of Technology Zrich ETHZ

              Version:

      Purpose: Manipulations of bits in bytes (8b) or words (16b)
              
      Remarks machine dependent!
              
      Programming

           Programming and Implementation
              O. Roth             18. 2.89
              
          Swiss Federal Institute of Technology Zurich
          ETH-Zentrum
          CH-8092 Zurich
          Switzerland

          Last revision:   09.03.89    or

  *******************************************************************)

  FROM SYSTEM IMPORT BYTE, WORD, SHIFT;



(* ------------------------------------------------------------------- *)
(* Assignment and Conversion procedures for WordTYPE: *)


  PROCEDURE WordToBinStr( a: WORD;  VAR BinStr: BinSTR16 );
  (* convert a two byte variable to an array of bits.        *)
    VAR
      i: CARDINAL;
      x: WordTYPE;
  BEGIN
    x.w:= a;
    FOR i:= 0 TO 15 DO
      IF ( i IN x.s ) THEN  BinStr[15-i]:= '1';
      ELSE                  BinStr[15-i]:= '0';
      END(*IF*);
    END(*FOR*);
  END WordToBinStr;



  PROCEDURE BinStrToWord( BinStr: BinSTR16;  VAR a: WORD );
  (* convert an array of bits to a two byte variable.        *)
    VAR
      i: CARDINAL;
      x: WordTYPE;
  BEGIN
    FOR i:= 0 TO 15 DO
      IF BinStr[i] # '0' THEN  INCL( x.s, 15-i );
      ELSE                     EXCL( x.s, 15-i );
      END(*IF*);
      a:= x.w;
    END(*FOR*);
  END BinStrToWord;



  PROCEDURE WordToOctStr( a: WORD;  VAR OctStr: OctSTR6 );
  (* convert a two byte variable to an array of OctDIGIT.  *)
    VAR
      i : INTEGER;
      w : WordTYPE;
  BEGIN
    w.w:= a;
    FOR i:= HIGH(OctStr) TO 0 BY -1 DO
      OctStr[i]:= CHR( (w.d MOD 8)+ORD('0') );
      w.d:= w.d DIV 8;
    END(*FOR*);
  END WordToOctStr;


  PROCEDURE OctStrToWord( OctStr: OctSTR6;  VAR a: WORD );
  (* convert an array of OctDIGIT to a two byte variable.  *)
    VAR
      i: INTEGER;
      w: WordTYPE;
  BEGIN
    w.d:= 0;
    FOR i:= 0 TO HIGH(OctStr) DO
      w.d:= w.d*8 + ORD(OctStr[i])-ORD('0');
    END(*FOR*);
    a:= w.w;
  END OctStrToWord;


  PROCEDURE WordToHexStr( a: WORD;  VAR HexStr: HexSTR4 );
  (* convert a two byte variable to an array of HexDIGIT.        *)
    VAR
      i, j,
      hexd : CARDINAL;
      x    : WordTYPE;
  BEGIN
    x.w:= a;
    FOR j:= 0 TO 3 DO
      hexd:= 0;
      FOR i:= 0 TO 3 DO
        IF ( (j*4 +i) IN x.s ) THEN   hexd:= hexd+Pow2[i];    END(*IF*);
      END(*FOR*);
      HexStr[3-j]:= HexDIGIT[hexd];
    END(*FOR*);
  END WordToHexStr;



  PROCEDURE HexOrd( HexChar: CHAR ) : CARDINAL;
    VAR  i: CARDINAL;
  BEGIN
    i:= 0;
    WHILE ( HexChar # HexDIGIT[i] ) DO
      INC( i );
    END(*WHILE*);
    RETURN i; 
  END HexOrd;


  PROCEDURE HexStrToWord( HexStr: HexSTR4;  VAR a: WORD );
  (* convert an array of HexDIGIT to a two byte variable.        *)
    VAR  x: WordTYPE;
  BEGIN
    x.d:=  HexOrd(HexStr[0]) *Pow2[12]
          +HexOrd(HexStr[1]) *Pow2[ 8]
          +HexOrd(HexStr[2]) *Pow2[ 4]
          +HexOrd(HexStr[3]) *Pow2[ 0];
    a:= x.w;
  END HexStrToWord;



(* ------------------------------------------------------------------- *)
(* BitManipulations for WordTYPE: *)


  PROCEDURE ShiftLeftW( VAR a: WORD; n : CARDINAL );
  (* shifts all bits of a n places to the left. *)
  BEGIN
    a:= SHIFT( a, n );
  END ShiftLeftW;



  PROCEDURE CircShiftLeftW( VAR a: WORD; n : CARDINAL );
  (* shifts all bits of a n places to the left, overflowing bits
     are reentered at right. *)
    VAR
      i: CARDINAL;
      x: WordTYPE;
  BEGIN
    x.w:= a;
    FOR i:= 1 TO n DO
      IF (15 IN x.s) THEN
        x.s:= SHIFT( x.s, 1 );
        INCL( x.s, 0 );
      ELSE
        x.s:= SHIFT( x.s, 1 );
      END(*IF*);
    END(*FOR*);
    a:= x.w;
  END CircShiftLeftW;



  PROCEDURE ShiftRightW( VAR a: WORD; n : CARDINAL );
  (* shifts all bits of a n places to the right. *)
    VAR  i: INTEGER;
  BEGIN
    i:= n;
    a:= SHIFT( a, -i );
  END ShiftRightW;



  PROCEDURE CircShiftRightW( VAR a: WORD; n : CARDINAL );
  (* shifts all bits of a n places to the right, overflowing bits
     are reentered at left. *)
    VAR
      i: CARDINAL;
      x: WordTYPE;
  BEGIN
    x.w:= a;
    FOR i:= 1 TO n DO
      IF ODD( x.d ) THEN      (* lowest bit was set      *)
        x.s:= SHIFT( x.s, -1 );
        INCL( x.s, 15 );      (* so set highest bit now. *)
      ELSE
        x.s:= SHIFT( x.s, -1 );
      END(*IF*);
    END(*FOR*);
    a:= x.w;
  END CircShiftRightW;



(* BitOperations for WordTYPE: *)


  PROCEDURE NotW( a: WORD ) : WORD;
  (* sets all bits of a to the opposite state. *)
    VAR  x: WordTYPE;
  BEGIN
    x.w:= a;
    x.d:= (MAX(CARDINAL) - x.d); (* equals: XorW( MAX(CARDINAL), x ); *)
    RETURN x.w;
  END NotW;



  PROCEDURE AndW( a, b : WORD ) : WORD;
  (* equals the logical operation AND on the two variables a and b. *)
    VAR
      r, x, y : WordTYPE;
  BEGIN
    x.w:= a;
    y.w:= b;
    r.s:= ( x.s * y.s );
    RETURN r.w;
  END AndW;



  PROCEDURE OrW( a, b : WORD ) : WORD;
  (* equals the logical operation OR on the two variables a and b. *)
    VAR
      r, x, y : WordTYPE;
  BEGIN
    x.w:= a;
    y.w:= b;
    r.s:= ( x.s + y.s );
    RETURN r.w;
  END OrW;



  PROCEDURE XorW( a, b : WORD ) : WORD;
  (* equals the logical operation XOR on the two variables a and b. *)
    VAR
      r, x, y : WordTYPE;
  BEGIN
    x.w:= a;
    y.w:= b;
    r.s:= ( x.s / y.s );
    RETURN r.w;
  END XorW;




(* ------------------------------------------------------------------- *)
(* Assignment and Conversion procedures for OneByteTYPE: *)


  PROCEDURE ByteToBinStr( a: BYTE;  VAR BinStr: BinSTR8 );
  (* convert a byte variable to an array of bits.      *)
    VAR
      i    : CARDINAL;
      x    : WordTYPE;
      bstr : BinSTR16;
  BEGIN
    x.b[0]:= 0C;
    x.b[1]:= a;
    WordToBinStr( x, bstr );
    FOR i:= 8 TO 15 DO  BinStr[i-8]:= bstr[i];  END(*FOR*);
  END ByteToBinStr;



  PROCEDURE BinStrToByte( BinStr: BinSTR8;  VAR a: BYTE );
  (* convert an array of bits to a byte variable.      *)
    VAR
      i    : CARDINAL;
      x    : WordTYPE;
      bstr : BinSTR16;
  BEGIN
    FOR i:= 0 TO  7 DO  bstr[i]:= '0';         END(*FOR*); (* not necessary but clearer. *)
    FOR i:= 8 TO 15 DO  bstr[i]:= BinStr[i-8]; END(*FOR*);
    BinStrToWord( bstr, x );
    a:= x.b[1];
  END BinStrToByte;



  PROCEDURE ByteToOctStr( a: BYTE;  VAR OctStr: OctSTR3 );
  (* convert a byte variable to an array of OctDIGIT.  *)
    VAR
      i : INTEGER;
      w : WordTYPE;
  BEGIN
    w.b[0]:= 0C;  w.b[1]:= a;
    FOR i:= HIGH(OctStr) TO 0 BY -1 DO
      OctStr[i]:= CHR( (w.d MOD 8)+ORD('0') );
      w.d:= w.d DIV 8;
    END(*FOR*);
  END ByteToOctStr;


  PROCEDURE OctStrToByte( OctStr: OctSTR3;  VAR a: BYTE );
  (* convert an array of OctDIGIT to a byte variable.  *)
    VAR
      i: INTEGER;
      w: WordTYPE;
  BEGIN
    w.d:= 0;
    FOR i:= 0 TO HIGH(OctStr) DO
      w.d:= w.d*8 + ORD(OctStr[i])-ORD('0');
    END(*FOR*);
    a:= w.b[1];
  END OctStrToByte;



  PROCEDURE ByteToHexStr( a: BYTE;  VAR HexStr: HexSTR2 );
  (* convert a byte variable to an array of HexDIGIT.  *)
    VAR
      x    : WordTYPE;
      hstr : HexSTR4;
  BEGIN
    x.b[0]:= 0C; (* not necessary but clearer. *)
    x.b[1]:= a;
    WordToHexStr( x, hstr );
    HexStr[0]:= hstr[2];   HexStr[1]:= hstr[3];
  END ByteToHexStr;



  PROCEDURE HexStrToByte( HexStr: HexSTR2;  VAR a: BYTE );
  (* convert an array of HexDIGIT to a byte variable.  *)
    VAR
      x    : WordTYPE;
      hstr : HexSTR4;
  BEGIN
    hstr[0]:= '0';   hstr[1]:= '0';   hstr[2]:= HexStr[0];  hstr[3]:= HexStr[1];
    HexStrToWord( hstr, x );
    a:= x.b[1];
  END HexStrToByte;



(* ------------------------------------------------------------------- *)
(* BitManipulations for OneByteTYPE: *)


  PROCEDURE ShiftLeftB( VAR a: BYTE; n : CARDINAL );
  (* shifts all bits of a n places to the left. *)
    VAR
      i, j: CARDINAL;
  BEGIN
    a:= SHIFT( a, n );
    (* alternatively: j:= ORD(a); INC( j, j ); or as well j*2 . *)
  END ShiftLeftB;



  PROCEDURE CircShiftLeftB( VAR a: BYTE; n : CARDINAL );
  (* shifts all bits of a n places to the left, overflowing bits
     are reentered at right. *)
    VAR
      i, j: CARDINAL;
      x   : WordTYPE;
  BEGIN
    x.b[0]:= 0C;   x.b[1]:= a;
    FOR i:= 1 TO n DO
      IF ( 7 IN x.s) THEN  (* highest bit was set    *)
        x.s:= SHIFT( x.s, 1 );
        INCL( x.s, 0 );    (* so set lowest bit now. *)
      ELSE
        x.s:= SHIFT( x.s, 1 );
      END(*IF*);
    END(*FOR*);
    a:= x.b[1];
  END CircShiftLeftB;



  PROCEDURE ShiftRightB( VAR a: BYTE; n : CARDINAL );
  (* shifts all bits of a n places to the right. *)
    VAR i: INTEGER;
  BEGIN
    i:= n;
    a:= SHIFT( a, -i );
  END ShiftRightB;



  PROCEDURE CircShiftRightB( VAR a: BYTE; n : CARDINAL );
  (* shifts all bits of a n places to the right, overflowing bits
     are reentered at left. *)
    VAR
      i: CARDINAL;
      x: WordTYPE;
  BEGIN
    x.b[0]:= 0C;  x.b[1]:= a;
    FOR i:= 1 TO n DO
      IF ODD(x.d) THEN         (* if lowest bit was set   *)
        x.s:= SHIFT( x.s, -1 );
        INCL( x.s, 7 );        (* so set highest bit now. *)
      ELSE
        x.s:= SHIFT( x.s, -1 );
      END(*IF*);
    END(*FOR*);
    a:= x.b[1];
  END CircShiftRightB;



(* BitOperations for OneByteTYPE: *)


  PROCEDURE NotB( a: BYTE ) : BYTE;
  (* sets all bits of a to the opposite state. *)
    VAR x: WordTYPE;
  BEGIN
    x.b[0]:= 0C;
    x.b[1]:= a;
    x.d:= MAX(CARDINAL) - x.d;
    a:= x.b[1];
    RETURN a;
  END NotB;



  PROCEDURE AndB( a, b : BYTE ) : BYTE;
  (* equals the logical operation AND on the two variables a and b. *)
    VAR
      x, y, r: WordTYPE;
  BEGIN
    x.b[0]:= 0C;  x.b[1]:= a;
    y.b[0]:= 0C;  y.b[1]:= b;
    r.w:= AndW( x, y );
    RETURN r.b[1];
  END AndB;



  PROCEDURE OrB( a, b : BYTE ) : BYTE;
  (* equals the logical operation OR on the two variables a and b. *)
    VAR
      x, y, r: WordTYPE;
  BEGIN
    x.b[0]:= 0C;  x.b[1]:= a;
    y.b[0]:= 0C;  y.b[1]:= b;
    r.w:= OrW( x, y );
    RETURN r.b[1];
  END OrB;



  PROCEDURE XorB( a, b : BYTE ) : BYTE;
  (* equals the logical operation XOR on the two variables a and b. *)
    VAR
      x, y, r: WordTYPE;
  BEGIN
    x.b[0]:= 0C;  x.b[1]:= a;
    y.b[0]:= 0C;  y.b[1]:= b;
    r.w:= XorW( x, y );
    RETURN r.b[1];
  END XorB;



(* ------------------------------------------------------------------- *)


BEGIN (* Bits *)

  HexDIGIT:= '0123456789ABCDEF';

  Pow2[ 0]:=    1;   Pow2[ 1]:=    2;   Pow2[ 2]:=    4;   Pow2[ 3]:=    8;
  Pow2[ 4]:=   16;   Pow2[ 5]:=   32;   Pow2[ 6]:=   64;   Pow2[ 7]:=  128;
  Pow2[ 8]:=  256;   Pow2[ 9]:=  512;   Pow2[10]:= 1024;   Pow2[11]:= 2048;
  Pow2[12]:= 4096;   Pow2[13]:= 8192;   Pow2[14]:=16384;   Pow2[15]:=32768;

END Bits.
