IMPLEMENTATION MODULE ComplexLib;
  (******************************************************************)
  (*                                                                *)
  (*        MODULE: ComplexLib,         Version   0.5 / 002         *)
  (*                                                                *)
  (*        Purpose: This Module offers different basic operation   *)
  (*                 on complex variables.                          *)
  (*                                                                *)
  (*        Implementation:  For Apple Macintosh                    *)
  (*                         Version fr  MacMETH V2.0              *)
  (*                         based on DM 0.4                        *)
  (*                                                                *)
  (*        Programming:     A.Itten                                *)
  (*                         C E L T I A                            *)
  (*                         ETH-Zentrum                            *)
  (*                         CH-8092 Zrich                         *)
  (*                                                                *)
  (*        LAST MODIFICATION :  12.07.91                           *)
  (******************************************************************)

  FROM ExtMathLib IMPORT EXTREAL,
                         ExtAdd, ExtSub, ExtMul, ExtDiv, ExtNeg, ExtAbs,
                         ExtSqrt, ExtSin, ExtCos, ExtArcTan, ExtExp,
                         RealToExt, ExtToReal;
  FROM DMMessages IMPORT DoInform;

  PROCEDURE SetCom(VAR c: Complex; re,im: REAL);
  BEGIN
    RealToExt(re,c.re);
    RealToExt(im,c.im);
  END SetCom;
    
  PROCEDURE GetCom(c: Complex; VAR re,im: REAL);
  BEGIN
    ExtToReal(c.re,re);
    ExtToReal(c.im,im);
  END GetCom;

  (* basic operations on the type Complex *)

  PROCEDURE KartToPol(x:Complex; VAR betr,phi: REAL);
  CONST halfPi = 1.570796327;
  VAR y1,y2 : EXTREAL;
      r,i : REAL;
  BEGIN
    WITH x DO
      ExtMul(re,re,y1); ExtMul(im,im,y2);
      ExtAdd(y1,y2,y1);
      ExtSqrt(y1,y2);
      ExtToReal(y2,betr);
      GetCom(x,r,i);
      IF r # 0.0 THEN
        ExtDiv(im,re,y1);
        ExtArcTan(y1,y2);
        ExtToReal(y2,phi);
      ELSIF i >= 0.0 THEN 
        phi:= halfPi;
      ELSE
        phi:= - halfPi;
      END; (* IF *)
    END; (* WITH *)
  END KartToPol;
   
  PROCEDURE PolToKart(betr,phi: REAL; VAR x:Complex);
  VAR y, eBetr,ePhi: EXTREAL;
  BEGIN
    RealToExt(betr,eBetr);
    RealToExt(phi,ePhi);
    WITH x DO
      ExtCos(ePhi,y);
      ExtMul(eBetr,y,re);
      ExtSin(ePhi,y);
      ExtMul(eBetr,y,im);
    END; (* WITH *)
  END PolToKart;
   
  PROCEDURE MultCom(a,b : Complex; VAR z: Complex);
  VAR y1,y2 : EXTREAL;
  BEGIN
    WITH z DO
      ExtMul(a.re,b.re,y1);   (* re:= a.re * b.re - a.im * b.im; *)
      ExtMul(a.im,b.im,y2);
      ExtSub(y1,y2,z.re); 
      ExtMul(a.re,b.im,y1);   (* im:= a.re * b.im + a.im * b.re; *)
      ExtMul(a.im,b.re,y2);
      ExtAdd(y1,y2,z.im);
    END (* WITH *)
  END MultCom;
   
  PROCEDURE  DivCom(a,b: Complex; VAR z: Complex);
  VAR y1,y2,den : EXTREAL;
  BEGIN
    WITH z DO
      ExtMul(b.re,b.re,y1);  (* den := b.re * b.re + b.im * b.im *)
      ExtMul(b.im,b.im,y2);
      ExtAdd(y1,y2,den);
      ExtMul(a.re,b.re,y1);  (* re:=(a.re*b.re+a.im*b.im)/den *)
      ExtMul(a.im,b.im,y2);
      ExtAdd(y1,y2,y2); ExtDiv(y2,den,z.re);
      ExtMul(a.im,b.re,y1);  (* im:=(a.im*b.re-a.re*b.im)/den *)
      ExtMul(a.re,b.im,y2);
      ExtSub(y1,y2,y2); ExtDiv(y2,den,z.im);
    END
  END DivCom;
   
  PROCEDURE  AddCom(a,b: Complex; VAR z: Complex);
  BEGIN
    ExtAdd(a.re,b.re,z.re);
    ExtAdd(a.im,b.im,z.im);
  END AddCom;
   
  PROCEDURE  SubCom(a,b: Complex; VAR z: Complex);
  BEGIN
    ExtSub(a.re,b.re,z.re);
    ExtSub(a.im,b.im,z.im);
  END SubCom;
   
  PROCEDURE  NegCom(VAR z : Complex);
  BEGIN
    ExtNeg(z.re,z.re);
    ExtNeg(z.im,z.im);
  END NegCom;

  PROCEDURE  ExpCom(VAR z: Complex);
  VAR betr,y : EXTREAL;
  BEGIN
    ExtExp(z.re,betr);
    ExtCos(z.im,y);
    ExtMul(betr,y,z.re);
    ExtSin(z.im,y);
    ExtMul(betr,y,z.im);
  END ExpCom;
   
  PROCEDURE  SkalCom(VAR z: Complex; k : REAL);
  VAR ek: EXTREAL;
  BEGIN
    RealToExt(k,ek);
    ExtMul(z.re,ek,z.re);
    ExtMul(z.im,ek,z.im);
  END SkalCom;
   
  PROCEDURE RandomCom(VAR a:Complex);
  BEGIN
    DoInform(100,"ComplexLib","RandomCom","");
  END RandomCom;
   
  PROCEDURE SetToZero(VAR a:Complex);
  BEGIN
    a:= zero;
  END SetToZero;

BEGIN  (* ComplexLib *)
  SetCom(one,1.0,0.0);
  SetCom(zero,0.0,0.0);
  SetCom(imagUnit,0.0,1.0);
END ComplexLib.
