IMPLEMENTATION MODULE Curves3D;

  (*

        Implementation and Revisions:
        ============================

        Author  Date        Description
        ------  ----        -----------

        mu      25/03/86    First implementation MacLogiomo
        mu      24/07/86    show wait symbol
        af      12/06/87    Mac3DGraphics to DM3DGraphics and DM 0.5
	jth	27/01/93    renamed DM3DCurves to Curves3D

  *)
	
FROM Graphics3D    IMPORT Point3D, Convert3DTo2D;
FROM DMWindowIO	   IMPORT SetUCPen, UCLineTo, PlotSym;
FROM DMMaster	   IMPORT ShowWaitSymbol, HideWaitSymbol;

	
VAR 
  UpdateStore: RECORD
		 new:   [0..nRun-1]; (*pointer to the new run*)
		 curve: ARRAY [0..nRun-1] OF
			  RECORD
			    pos: [0..nVal]; (*pointer to the  
						last written value*)
			    typ: ProjectionSet;
			    val: ARRAY[0..nVal] OF Point3D;
			  END;
	       END;
	       
  currentProjection: ProjectionSet;
  storageOn        : BOOLEAN;
  plotSymbols	   : ARRAY Projections OF CHAR;
  previousPoint,
  newPoint	   : Point3D;
    
  
PROCEDURE SelectSymbol(theProjection: Projections; symbol: CHAR); 
BEGIN (*SelectSymbol*)
  plotSymbols[theProjection]:= symbol;
END SelectSymbol;


PROCEDURE ClearUpdateStore;
  VAR i: CARDINAL;
BEGIN (*ClearUpdateStore*)
  WITH UpdateStore DO
    new:= 0; 
    FOR i:= 0 TO nRun-1 DO 
      WITH curve[i] DO pos:= 0; typ:= ProjectionSet{}  END;
    END; (*FOR*)
  END; (*WITH*)
END ClearUpdateStore;


PROCEDURE StartNewCurve(projection:ProjectionSet; firstPoint: Point3D);
  (*This procedure stores the display for a curve and its first point.	*) 
BEGIN (*StartNewCurve*)
  currentProjection:= projection;
  previousPoint:= firstPoint;
  IF storageOn THEN
    WITH UpdateStore DO
      new:= (new+1) MOD nRun; 
      WITH curve[new] DO
	pos:= 0;
	typ:= projection;
	val[pos]:= firstPoint;
      END; (*WITH*)
    END; (*WITH*)
  END; (*IF*)
END StartNewCurve;


PROCEDURE StoreValue(P: Point3D);
  (*This procedure stores P at the given position *)
BEGIN (*StoreValue*)
  WITH UpdateStore.curve[UpdateStore.new] DO
    (* vorsorgen, dass Arraygrenze nicht ueberschritten wird*)
    IF pos<nVal THEN INC(pos); val[pos]:= P END;
  END; (*WITH*)
END StoreValue;


PROCEDURE PlotTo3D(P: Point3D);
  (* This procedure plots all projections contained in display from 
     the last point (defined by the last call of PlotTo3D, or by calling
     StartNewCurve) to P. At the end of each line, the symbol
     defined by default or by 'SelectSymbol' is plotted. 		*)

CONST
  maxInt = 31000.0; (*groesster Integer, in real-Format*)
  (*dies sollte mit einer zukuenftigen Version von MacWindows nicht mehr
    noetig sein								*)

VAR
  i: Projections; previousDrawPoint, newDrawPoint: Point3D;

  PROCEDURE Line3DfromTo(previous3DPoint, new3DPoint: Point3D);
    VAR prevU, prevV, newU, newV: REAL; (*u and v-coordinates of previous
    					  and new 2D points		*)
  BEGIN
    Convert3DTo2D(previous3DPoint, prevU, prevV);
    Convert3DTo2D(new3DPoint,      newU,  newV);

  (*verhindert Integer Overflow, sollte aber mit einer zukuenftigen
    Version von MacWindows nicht mehr noetig sein:			*)
    IF     (ABS(prevU)<maxInt) AND (ABS(prevV)<maxInt)
        AND (ABS(newU)<maxInt) AND (ABS(newV)<maxInt)
  (*.....*)	
    THEN   
      SetUCPen(prevU, prevV); 
      UCLineTo(newU,  newV);
    END; (*IF*)
  END Line3DfromTo;

BEGIN (*PlotTo3D*)
  IF storageOn THEN StoreValue(P) END;
  newPoint:= P;
  FOR i:= xyPlane TO spacial DO
    IF i IN UpdateStore.curve[UpdateStore.new].typ THEN
      previousDrawPoint:= previousPoint;
      newDrawPoint     := newPoint;
      CASE i OF  
	  xyPlane : previousDrawPoint.z := 0.0; (*Projection on xy-plane*)
		    newDrawPoint.z      := 0.0;
		    Line3DfromTo(previousDrawPoint,newDrawPoint);
	| xzPlane : previousDrawPoint.y := 0.0; (*Projection on xz-plane*)
		    newDrawPoint.y      := 0.0;
		    Line3DfromTo(previousDrawPoint,newDrawPoint);
	| yzPlane : previousDrawPoint.x := 0.0; (*Projection on yz-plane*)
		    newDrawPoint.x      := 0.0;
		    Line3DfromTo(previousDrawPoint,newDrawPoint);
	| spacial : Line3DfromTo(previousDrawPoint,newDrawPoint);
      END; (*CASE*);
      PlotSym(plotSymbols[i]);
    END; (*IF*)
  END; (*FOR*);    
  previousPoint:= newPoint;
END PlotTo3D;


PROCEDURE ReplotAll;
  VAR i, j, saveNew: CARDINAL; saveStatus: BOOLEAN;
BEGIN (*ReplotAll*)
  ShowWaitSymbol;
    saveStatus:= storageOn;
    StorageOff;
    saveNew:= UpdateStore.new;
    FOR i:= 0 TO nRun-1 DO
      UpdateStore.new:= i;
      WITH UpdateStore.curve[UpdateStore.new] DO
	IF pos>0 THEN
	  previousPoint:= val[0];
	  FOR j:= 1 TO pos DO PlotTo3D(val[j]) END;
	END;(*IF*)
      END;(*WITH*)
    END;(*FOR*)
    UpdateStore.new:= saveNew;
    storageOn:= saveStatus;
  HideWaitSymbol;
END ReplotAll;


PROCEDURE GetCurrentProjection(): ProjectionSet;
BEGIN 
  RETURN currentProjection;
END GetCurrentProjection;

  
PROCEDURE StorageOff;
BEGIN 
  storageOn:= FALSE;
END StorageOff;
  
  
PROCEDURE StorageOn;
BEGIN
  storageOn:= TRUE;
END StorageOn;


PROCEDURE InitSymbols;
  VAR i: Projections;
BEGIN
  FOR i:= xyPlane TO spacial DO plotSymbols[i]:=0C END;
END InitSymbols;


BEGIN (*Curves3D.*)  
  currentProjection:= ProjectionSet{};
  storageOn:= TRUE;
  InitSymbols;
  ClearUpdateStore;
END Curves3D.