IMPLEMENTATION MODULE ExtMathLib (* A.I.  14.11.86 *);

  FROM 		SYSTEM    IMPORT ADDRESS,ADR;
  
  CONST  FP68K        = 0A9EBH;        ELEM68K      = 0A9ECH;
		
	 OpCodeAdd    = 00000H;        OpCodeExp    = 00008H; 
	 OpCodeSub    = 00002H;        OpCodeLn     = 00000H;
	 OpCodeMul    = 00004H;        OpCodeSin    = 00018H;
	 OpCodeDiv    = 00006H;        OpCodeCos    = 0001AH;
	 OpCodeRem    = 0000CH;        OpCodeArcTan = 0001EH;
	 OpCodeSqrt   = 00012H;
	 OpCodeNeg    = 0000DH;
	 OpCodeAbs    = 0000FH;
  
	 ConvToExt    = 0000EH;
	 ConvFromExt  = 00010H;
  
	 SrcFmtInt    = 02000H;
	 SrcFmtLongInt= 02800H;
	 SrcFmtSingle = 01000H;
	 SrcFmtDouble = 00800H;
	 DstFmtSingle = 08000H;
	 DstFmtDouble = 04000H;
  
	 SetEnviron   = 00001H;
	 GetEnviron   = 00003H;
	 
	 RoundUp      = 0DH;
	 RoundDown    = 0EH;       
  
       
  PROCEDURE Call1FP68K(Src : ADDRESS; OpWord : CARDINAL);
  
    CODE FP68K;
  
  PROCEDURE Call2FP68K(Src,Dst : ADDRESS; OpWord : CARDINAL);
  
    CODE FP68K;
  
  PROCEDURE Call1ELEM68K(Src : ADDRESS; OpWord : CARDINAL);
  
    CODE ELEM68K;
  
  
  (* same Procedures as is MathLib and LongMathLib *)
  
  PROCEDURE ExtSqrt(x: EXTREAL; VAR res: EXTREAL);
    BEGIN
      res:= x;
      Call1FP68K(ADR(res),OpCodeSqrt);
    END ExtSqrt;
  
  PROCEDURE ExtExp(x: EXTREAL; VAR res: EXTREAL);
    BEGIN
      res:= x;
      Call1ELEM68K(ADR(res),OpCodeExp)
    END ExtExp;
  
  PROCEDURE ExtLn(x: EXTREAL; VAR res: EXTREAL);
    BEGIN
      res:= x;
      Call1ELEM68K(ADR(res),OpCodeLn)
    END ExtLn;
  
  PROCEDURE ExtSin(x: EXTREAL; VAR res: EXTREAL);
    BEGIN
      res:= x;
      Call1ELEM68K(ADR(res),OpCodeSin)
    END ExtSin;
  
  PROCEDURE ExtCos(x: EXTREAL; VAR res: EXTREAL);
    BEGIN
      res:= x;
      Call1ELEM68K(ADR(res),OpCodeCos)
    END ExtCos;
  
  PROCEDURE ExtArcTan(x: EXTREAL; VAR res: EXTREAL);
    BEGIN
      res:= x;
      Call1ELEM68K(ADR(res),OpCodeArcTan)
    END ExtArcTan;
  
  PROCEDURE ExtReal(x: LONGINT; VAR res: EXTREAL);
    BEGIN
      Call2FP68K(ADR(x),ADR(res),ConvToExt + SrcFmtLongInt)
    END ExtReal;
  
  PROCEDURE ExtEntier(x: EXTREAL; VAR res: LONGINT );
    VAR old,
	env : BITSET;
    
    BEGIN
      Call1FP68K(ADR(old),GetEnviron);
      env := old - {RoundUp} + {RoundDown};
      Call1FP68K(ADR(env),SetEnviron);
      Call2FP68K(ADR(x),ADR(res),ConvFromExt + SrcFmtLongInt);
      Call1FP68K(ADR(old),SetEnviron)
    END ExtEntier;
  
  
  (* Conversions to and from REAL and LONGREAL *)
  
  PROCEDURE LongToExt(x: LONGREAL; VAR e: EXTREAL);
    BEGIN
      Call2FP68K(ADR(x),ADR(e),ConvToExt + SrcFmtDouble)
    END LongToExt;
    
  PROCEDURE ExtToLong(e: EXTREAL; VAR res: LONGREAL);
    BEGIN
      Call2FP68K(ADR(e),ADR(res),ConvFromExt + SrcFmtDouble)
    END ExtToLong;
    
  PROCEDURE RealToExt(x: REAL; VAR res: EXTREAL);
    BEGIN
      Call2FP68K(ADR(x),ADR(res),ConvToExt + SrcFmtSingle)
    END RealToExt;
  
  PROCEDURE ExtToReal(e: EXTREAL; VAR res:     REAL);
    BEGIN
      Call2FP68K(ADR(e),ADR(res),ConvFromExt + SrcFmtSingle)
    END ExtToReal;
  
  (* basic Procedures *)
  
  PROCEDURE ExtAdd(a,b: EXTREAL; VAR res: EXTREAL);
    BEGIN
      res:= a;
      Call2FP68K(ADR(b),ADR(res),OpCodeAdd)
    END ExtAdd;
      
  PROCEDURE ExtSub(a,b: EXTREAL; VAR res: EXTREAL);
    BEGIN
      res:= a;
      Call2FP68K(ADR(b),ADR(res),OpCodeSub)
    END ExtSub;
  
  PROCEDURE ExtMul(a,b: EXTREAL; VAR res: EXTREAL);
    BEGIN
      res:= a;
      Call2FP68K(ADR(b),ADR(res),OpCodeMul)
    END ExtMul;
  
  PROCEDURE ExtDiv(a,b: EXTREAL; VAR res: EXTREAL);
    BEGIN
      res:= a;
      Call2FP68K(ADR(b),ADR(res),OpCodeDiv)
    END ExtDiv;
  
  
  PROCEDURE ExtAbs(e: EXTREAL; VAR res: EXTREAL);
    BEGIN
      res:= e;
      Call1FP68K(ADR(res),OpCodeAbs)
    END ExtAbs;
  
  PROCEDURE ExtNeg(e: EXTREAL; VAR res: EXTREAL);
    BEGIN
      res:= e;
      Call1FP68K(ADR(res),OpCodeNeg)
  END ExtNeg;

BEGIN (* ExtMathLib *)
END ExtMathLib.

