IMPLEMENTATION MODULE Graphics3D;

  (*

        Implementation and Revisions:
        ============================

        Author  Date        Description
        ------  ----        -----------

        mu      25/03/86    First implementation MacLogiomo
        mu      09/06/86    leaving cs-editor with extra-menu
        mu      10/09/86    DialogMachineTasks to DialogMachineTask
        af      12/06/87    Mac3DGraphics to DM3DGraphics and DM 0.5
        af		24/02/88	DM 1.0
        af		15/07/88	SetTermMenuText and GetTermMenuText added
        dg  	11/08/90    Adaptation to MacMETH 2.6.2: 
                            DMStrings.notAllocatedString imported
        dg  	09/01/91    Now imports from DMMathLib 
        dg  	13/03/92    Term proc deallocating menu-strings installed 
        af		20/09/92	Bug fixes in alerts
        jth		27/01/93    Renamed DM3DGraphics to Graphics3D

  *)
  
(* Subsequently, the abreveation 'CS' holds for coordinate system.      *)    

FROM DMSystem           IMPORT CurrentDMLevel, InstallTermProc;
FROM DMMathLib          IMPORT  Sqrt, Real, Entier, Ln;
FROM DMMaster           IMPORT  DialogMachineTask, RunDialogMachine;
FROM DMLanguage         IMPORT  Language, SetLanguage, CurrentLanguage;
FROM DMAlerts           IMPORT  ShowAlert, WriteMessage;
FROM DMMenus            IMPORT  AccessStatus, Marking, 
                                Menu, Command, UseMenu,
                                InstallCommand, InstallMenu, RemoveMenu, 
                                DisableDeskAccessories, 
                                EnableDeskAccessories,
                                InstallAliasChar;          

FROM DMWindowIO         IMPORT  
  (*Dragging*)
  ClickKind, PointClicked, RectClicked, GetCurMousePos, Drag, 
  GetLastMouseClick,                                 

  (*General output*) PaintMode, SetMode, pat, Color, GreyContent, 
  SetBackground, SetColor, GetColor, CellWidth, CellHeight, MaxCol, 
  MaxLn, BackgroundWidth, BackgroundHeight, EraseContent, 
  RedrawContent, SetClipping, RemoveClipping,                                 

  (*Text output*) WindowFont, AssignWindowFont, SetPos, Write, 
  WriteString, WriteInt,                                 

  (*Pixel graphics*) SetPen, LineTo, Circle,                                 

  (*plotting in user coordinates*) ScaleUC, UCFrame, 
  EraseUCFrameContent, SetUCPen, UCLineTo, UCDot, ConvertPointToUC, 
  ConvertUCToPoint;                                 


FROM DMWindows          IMPORT  
  RectArea;
  
FROM DMStrings IMPORT
  String, NewString, PutString, GetString, ConcatString,
  DisposeString, notAllocatedString;

FROM SYSTEM IMPORT VAL;


CONST
  AxisTickLength        = 4.0; (*real pixels*)
  radius                = 6; (*radius for highlight circles in DefCS*)
  OriginProhibited      = 8; (*minimum distance of origin from CS frame,*)
                             (*defined by f                             *)
 
TYPE
  ErrorTypes            = (badCSMaxima,badCSPoints,badCSorigin);
  CSeditType            = (AxisChange,ScaleChange,OriginChange);
  Matrix2x2             = ARRAY [1..2] OF ARRAY [1..2] OF REAL;
  Vector3D              = RECORD x,y,z: REAL END;
  Vector2D              = RECORD u,v: REAL END;
  Point2D               = RECORD u,v: REAL END;
  AxisIDenumerator      = (X,Y,Z);
  AxisIdentifier        = [X..Z];
  CSFrameType           = RECORD uMin,uMax,vMin,vMax:REAL END;
  CSAxisType            = RECORD
                            F,S   : Point2D; (*Frame, Scale point*)
                            units, maxUnits: REAL;    
                          END;
  CSDefPointsType       = RECORD
                            O     : Point2D;    (*origin*)
                            Axis  : ARRAY AxisIdentifier OF CSAxisType;
                          END;
  CSDefVectorsType      = RECORD 
                            offsetVector: Vector2D;
                            UnityVector : ARRAY AxisIdentifier OF Vector2D;
                          END;
  CSAxisLabelsType      = ARRAY AxisIdentifier OF AxisLabelString;
  CSiType               = RECORD          (*CS implementation Type*)
                            Frame     : CSFrameType;
                            DefPoints : CSDefPointsType;
                            DefVectors: CSDefVectorsType;
                            AxisLabels: CSAxisLabelsType
                          END;
                          
VAR
  CSi           : CSiType;         (*the coordinate system, for the     *)
                                   (*implementation module              *)
  currentCS     : CSType; (*stores parameters of current active CS      *)
  curMText,curMCmdText: String;
  curAliasChar: CHAR;
  NewDefPoints  : CSDefPointsType;
  graphField    : RectArea;
  a,(*reserved for the selected axis,must not be changed except in EditCS*)
  ax            : AxisIdentifier;
  CSedit        : CSeditType;
                  (*For Procedure 'AllowedMousePosition', AxisScaling:  *)
  d             : INTEGER;(*distance of labels from framelines(DrawCS)  *)
  editPointsOn  : BOOLEAN; (*TRUE: edit points highlighted by 
                             Procedure "ToggleEditPoints"               *)
  fu, fv        : REAL; (*coordinates of rotated Point F*)
  fuint, fvint  : INTEGER; (*coord. of rotated Point F, in window pixels*)
  ou, ov        : REAL; (*coordinates of rotated origin*)
  ouint, ovint  : INTEGER; (*coord. of rotated origin, in window pixels *)
  RotationMatrix: Matrix2x2;
  
  (*variables for extra menu to leave cs-editor:*)
  ExitMenu      : Menu;
  exitPos       : Command;
  exit          : BOOLEAN;
 
PROCEDURE BadCSMaximaP;
BEGIN
  CASE CurrentLanguage() OF
    English: 
    
WriteMessage(2,3,' Error in Procedure "InstallCS" of Graphics3D: ');
WriteMessage(4,3,"      CS.xMax, CS.yMax or CS.zMax out of"); 
WriteMessage(5,3,"      range (0.0...MAX(REAL)).");  

  | German : ;
    
WriteMessage(2,3,' Fehler in Prozedur "InstallCS" von Graphics3D: ');
WriteMessage(4,3,"      CS.xMax, CS.yMax oder CS.zMax ausserhalb");
WriteMessage(5,3,"      des Wertebereichs (0.0...MAX(REAL)).");  

  END; (*CASE*)
END BadCSMaximaP;


PROCEDURE BadCSPointsP;
BEGIN
  CASE CurrentLanguage() OF
    English: 
    
WriteMessage(2,3,' Error in Procedure "InstallCS" of Graphics3D: ');
WriteMessage(4,3,"      CS.xPoint, CS.yPoint or CS.zPoint doesn't lie on");  
WriteMessage(5,3,"      the boundary of the rectangle r.");  

  | German : ;
    
WriteMessage(2,3,' Fehler in Prozedur "InstallCS" von Graphics3D: ');
WriteMessage(4,3,"      CS.xPoint, CS.yPoint oder CS.zPoint liegt");
WriteMessage(5,3,"      nicht auf dem Rand des Rechtecks r.");  

  END; (*CASE*)
END BadCSPointsP;


PROCEDURE BadCSoriginP;
BEGIN
  CASE CurrentLanguage() OF
    English: 
    
WriteMessage(2,3,' Error in Procedure "InstallCS" of Graphics3D: ');
WriteMessage(4,3,"      CS.origin out of range:");  
WriteMessage(5,3,"          r.x+delta<= CS.origin.x<=r.x+r.w-delta.");  
WriteMessage(6,3,"          r.y+delta<= CS.origin.y<=r.y+r.h-delta.");  
WriteMessage(7,3,"          delta is a small positive constant.");  

  | German : ;
    
WriteMessage(2,3,' Fehler in Prozedur "InstallCS" von Graphics3D: ');
WriteMessage(4,3,"      CS.origin liegt ausserhalb des erlaubten Rechtecks:");
WriteMessage(5,3,"          r.x+delta<= CS.origin.x<=r.x+r.w-delta.");  
WriteMessage(6,3,"          r.y+delta<= CS.origin.y<=r.y+r.h-delta.");  
WriteMessage(7,3,"          delta ist eine kleine positive Konstante.");  

  END; (*CASE*)
END BadCSoriginP;

PROCEDURE Error(e: ErrorTypes);
BEGIN (*Error*)
  CASE e OF
    badCSMaxima:   ShowAlert(8,62, BadCSMaximaP);
  | badCSPoints:   ShowAlert(8,62, BadCSPointsP);
  | badCSorigin:   ShowAlert(8,62, BadCSoriginP);
  END; (*CASE*)
END Error;


PROCEDURE DefPointsToDefVectors;
BEGIN
  WITH CSi DO
    (*offset vector*)
    DefVectors.offsetVector.u:= DefPoints.O.u; 
    DefVectors.offsetVector.v:= DefPoints.O.v;
    (*unity vectors*)
    FOR ax:= X TO Z DO
      WITH DefPoints DO
        WITH Axis[ax] DO
          DefVectors.UnityVector[ax].u:= (S.u - O.u)/units; 
          DefVectors.UnityVector[ax].v:= (S.v - O.v)/units;
        END; (*WITH*)
      END; (*WITH*)
    END; (*FOR*)
  END; (*WITH*)
END DefPointsToDefVectors;
         
PROCEDURE GetUnits(maxUnits: REAL):REAL;
  (* Given  the origin,the endpoint F and maxunits (which must NOT be 
     zero) this procedcure calculates units.                            *)
  VAR l,i: INTEGER; units: REAL;
BEGIN (*GetUnits*)
  l:= Entier(Ln(maxUnits)/Ln(10.0));
        (* abgerundeter 10er-Log. von maxUnits   *)
  units:= 1.0;
  IF l>=1 THEN
    FOR i:= 1 TO l DO units:= units*10.0 END;
  ELSIF l<=-1 THEN
    FOR i:= 1 TO ABS(l) DO units:= units/10.0 END;
  END; (*IF*)
  IF units>maxUnits*10.0/11.0 THEN units:= units/10.0 END;
     (* units has the format 10^n, n entier. This number is 
        indicated as axistick on the axis. If units is to close 
        to the end of the axis, it will be divided by 10. *)
  RETURN units;
END GetUnits;


PROCEDURE InstallScalepoint(a: AxisIdentifier);
  (* Given the origin, the endpoint F of the axis, units and maxunits,
     this procedure calculates the coordinates of the scaling point S   *)
BEGIN (*InstallScalepoint*)
  WITH CSi.DefPoints DO
    WITH Axis[a] DO 
      S.u:= O.u + (F.u-O.u)*(units/maxUnits);
      S.v:= O.v + (F.v-O.v)*(units/maxUnits);
    END; (*WITH*)
  END; (*WITH*)
END InstallScalepoint;
         
         
PROCEDURE InstallCS(r: RectArea; CS: CSType);
  (* Installs a three-dimensional coodinate system defined by CS
     in the rectArea r.                                                 *)

  PROCEDURE CSok():BOOLEAN;
    VAR ok1, ok2, ok3: BOOLEAN;

    PROCEDURE PointOk(x,y:INTEGER):BOOLEAN;
      VAR withinFrame, onFrame: BOOLEAN;
    BEGIN (*PointOk*)
      withinFrame:= (x>=r.x) AND (x<=r.x+r.w) AND (y>=r.y) AND (y<=r.y+r.h);
      onFrame:= (x=r.x) OR (x=r.x+r.w) OR (y=r.y) OR (y=r.y+r.h);
      RETURN withinFrame AND onFrame;
    END PointOk;
    
  BEGIN (*CSok*)
    WITH CS DO
      ok1:= (xMax>0.0)       AND (yMax>0.0)       AND (zMax>0.0)      (*AND
            (xMax<MAX(REAL)) AND (yMax<MAX(REAL)) AND (zMax<MAX(REAL))   *);
        (*currently positive scaling supported only *)
      IF  NOT ok1 THEN Error(badCSMaxima) END;
      ok2:= PointOk(xPoint.x,xPoint.y) AND 
            PointOk(yPoint.x,yPoint.y) AND 
            PointOk(zPoint.x,zPoint.y);
      IF NOT ok2 THEN Error(badCSPoints) END;
      ok3:= (origin.x>=r.x + OriginProhibited)       AND 
            (origin.x<=r.x + r.w - OriginProhibited) AND 
            (origin.y>=r.y + OriginProhibited)       AND 
            (origin.y<=r.y + r.h - OriginProhibited);
      IF NOT ok3 THEN Error(badCSorigin) END;
    END; (*WITH*)
    RETURN ok1 AND ok2 AND ok3;
  END CSok;
  

  PROCEDURE AssignLabelText;
    VAR ok: BOOLEAN;
  BEGIN (*AssignLabelText*)
    WITH CSi DO 
      AxisLabels[X]:= CS.xLabel; 
      AxisLabels[Y]:= CS.yLabel; 
      AxisLabels[Z]:= CS.zLabel;
    END; (*WITH*)
  END AssignLabelText;
    
BEGIN (*InstallCS*)
  (*Scales, assigns labels, installs CSi.DefPoints and .Vectors*)
  IF CSok() THEN
    currentCS:= CS;
    graphField:= r;    
    WITH CSi.Frame DO
      uMin:= 0.0; vMin:= 0.0; 
      uMax:= Real(graphField.w); vMax:= Real(graphField.h);
      ScaleUC(graphField, uMin, uMax, vMin, vMax);
    END; (*WITH*)
    SetClipping(graphField);
    AssignLabelText;
    WITH CSi.DefPoints DO
      ConvertPointToUC(CS.origin.x, CS.origin.y, O.u, O.v);
      WITH Axis[X] DO
        ConvertPointToUC(CS.xPoint.x, CS.xPoint.y, F.u, F.v);
        maxUnits:= CS.xMax;
        units:= GetUnits(maxUnits);
        InstallScalepoint(X);
      END; (*WITH*)
      WITH Axis[Y] DO
        ConvertPointToUC(CS.yPoint.x, CS.yPoint.y, F.u, F.v);
        maxUnits:= CS.yMax;
        units:= GetUnits(maxUnits);
        InstallScalepoint(Y);
      END; (*WITH*)
      WITH Axis[Z] DO
        ConvertPointToUC(CS.zPoint.x, CS.zPoint.y, F.u, F.v);
        maxUnits:= CS.zMax;
        units:= GetUnits(maxUnits);
        InstallScalepoint(Z);
      END; (*WITH*)
    END; (*WITH*)
    DefPointsToDefVectors;
  END; (*IF*)
END InstallCS;


PROCEDURE GetCS(VAR CS: CSType);
BEGIN
  CS:= currentCS;
END GetCS;


PROCEDURE GetRotationMatrix(O,F: Point2D; VAR RotationMatrix: Matrix2x2);
  VAR r: REAL;
BEGIN
  (*This procedure calculates a matrix for a geometric rotation:
    Fixpoint of the rotation is the point (0.0,0.0) in UC.
    Angle so that the line OF is horizontal, O left of F        *)
    r:= Sqrt((F.u-O.u)*(F.u-O.u) + (F.v-O.v)*(F.v-O.v));
    IF r=0.0 THEN   (*no rotation*)
      RotationMatrix[1,1]:= 1.0;
      RotationMatrix[1,2]:= 0.0;
      RotationMatrix[2,1]:= 0.0;
      RotationMatrix[2,2]:= 1.0;
    ELSE
      RotationMatrix[1,1]:= -(O.u-F.u)/r; (*cos(rotAngle)*)
      RotationMatrix[1,2]:= -(O.v-F.v)/r; (*sin(rotAngle)*)
      RotationMatrix[2,1]:= -RotationMatrix[1,2];
      RotationMatrix[2,2]:=  RotationMatrix[1,1];
    END; (*IF*)
END GetRotationMatrix;
            
            
PROCEDURE Representation(RotationMatrix: Matrix2x2; u,v:REAL; 
                                                VAR fu,fv: REAL);
BEGIN
  fu:= RotationMatrix[1,1]*u + RotationMatrix[1,2]*v;
  fv:= RotationMatrix[2,1]*u + RotationMatrix[2,2]*v;
END Representation;

            
PROCEDURE InverseRepr(RotationMatrix: Matrix2x2; u,v:REAL; 
                                                VAR fu,fv: REAL);
BEGIN
  fu:=  RotationMatrix[1,1]*u - RotationMatrix[1,2]*v;
  fv:= -RotationMatrix[2,1]*u + RotationMatrix[2,2]*v;
END InverseRepr;


PROCEDURE DrawCS;

  PROCEDURE AxisTick(P: Point2D);
    VAR slope, du, delta: REAL; start,stop: Point2D;
  BEGIN 
    du:= CSi.DefPoints.O.u - P.u;
    IF du=0.0 THEN
      start.u:= P.u - AxisTickLength;
      stop.u := P.u + AxisTickLength;
      start.v:= P.v;
      stop.v := P.v;
    ELSE
      slope:= (CSi.DefPoints.O.v - P.v)/du;
      IF ABS(slope) > 3.0*AxisTickLength THEN
        start.u:= P.u - AxisTickLength;
        stop.u := P.u + AxisTickLength;
        start.v:= P.v;
        stop.v := P.v;
      ELSE
        delta:= AxisTickLength/(Sqrt(1.0+slope*slope));
        start.u:= P.u + slope*delta;
        stop.u := P.u - slope*delta;
        start.v:= P.v - delta;
        stop.v := P.v + delta;
      END (*IF*);
    END (*IF*);
    SetUCPen(start.u,start.v);
    UCLineTo(stop.u,stop.v);
  END AxisTick;
        
  PROCEDURE WriteLabel(P: Point2D; VAR label: ARRAY OF CHAR);
    VAR uOff, vOff: REAL; (*offset for writing of labels*)
        h,w,l: INTEGER; (*cellheight, -width, labellength*)
        O   : Point2D; (*origin*)
    
    PROCEDURE Length(VAR label:ARRAY OF CHAR):INTEGER;
      VAR i: INTEGER;
    BEGIN
      i:= 0;
      WHILE (i<=HIGH(label)) AND (label[i]<>0C) DO INC(i) END;
      RETURN i;  
    END Length;
    
    PROCEDURE OffsetFunction():REAL;
      VAR uOff, du ,dv ,mInvers: REAL;
    BEGIN
      dv:= O.v-P.v;
      WITH CSi.Frame DO
        IF dv=0.0 THEN
          uOff:= P.u;
        ELSE 
          mInvers:= (O.u-P.u)/dv;
          (*distinguish lower and upper zone:*)
          IF P.v<(vMax+vMin)/2.0 THEN
            uOff:= ((vOff+Real(h))*mInvers)+P.u;
          ELSE
            uOff:= (vOff*mInvers)+P.u;
          END; (*IF*)
        END; (*IF*)
        IF P.u<(uMax+uMin)/2.0 THEN
          IF uOff<P.u THEN uOff:= P.u END; (*uOff:= Max(uOff,P.u)*)
          uOff:= uOff+Real(2*d)-P.u;
        ELSE
          IF uOff>P.u THEN uOff:= P.u END;(*uOff:= Min:(uOff,P.u)*)
          uOff:= uOff-Real(2*d+l*w)-P.u;
        END; (*IF*)
      END; (*WITH*)
      RETURN uOff;
    END OffsetFunction;
    
    PROCEDURE StandardRange(u:REAL):BOOLEAN;
      VAR r: BOOLEAN;
    BEGIN
      WITH CSi.Frame DO
        r:= (u>=uMin+Real(l*w+2*d)) AND 
            (u<=uMax-Real(l*w+2*d));
      END; (*WITH*)
      RETURN r;
    END StandardRange;
    
    
  BEGIN (*WriteLabel*)
    l:= Length(label);
    h:= CellHeight(); w:= CellWidth(); O:= CSi.DefPoints.O;
    WITH CSi.Frame DO
      IF P.v >= vMax - Real(h+d) THEN
        vOff:= -Real(h+d);
        IF StandardRange(P.u) THEN
          IF O.u<=P.u THEN
            uOff:= Real(2*d) 
          ELSE 
            uOff:= Real(-2*d-l*w) 
          END; (*IF*)
        ELSE
          uOff:= OffsetFunction();
        END; (*IF*)
      ELSIF P.v <= vMin + Real(h+d+4) THEN
        vOff:= Real(d);
        IF StandardRange(P.u) THEN
          IF O.u<=P.u THEN
            uOff:= Real(2*d) 
          ELSE 
            uOff:= Real(-2*d-l*w) 
          END; (*IF*)
        ELSE
          uOff:= OffsetFunction();
        END; (*IF*)
      ELSIF P.u = uMin THEN
        uOff:= Real(2*d);
        IF O.v >= P.v THEN vOff:= Real(-h-d) ELSE vOff:= Real(d) END;
      ELSE (*P.u = uMax*)
        uOff:= Real(-2*d-l*w);
        IF O.v >= P.v THEN vOff:= Real(-h-d) ELSE vOff:= Real(d) END;
      END; (*IF*)
    END; (*WITH*)
    SetUCPen(P.u+uOff, P.v+vOff);
    WriteString(label);
  END WriteLabel;
    
  PROCEDURE WriteTickLabel;
    CONST xDistance = 4.0; yDistance = 2.0; yExtra = 2.0;
    VAR Matrix: Matrix2x2; du, dv, Surot, Svrot, w, h, startx, starty: REAL; 
        
    PROCEDURE Log(units: REAL): INTEGER;
      VAR log: INTEGER; u: REAL;
    BEGIN
      u:= units; log:= 0;
      IF u>1.0 THEN
        REPEAT u:= u/10.0; log:= log+1 UNTIL u<=1.0;
      ELSIF u<1.0 THEN
        REPEAT u:= u*10.0; log:= log-1 UNTIL u>=1.0;
      END; (*IF*)
      RETURN log;
    END Log;
      
      
  BEGIN (*WriteTickLabel*)
    (* Beschriftung der Striche mit dem Skalierwert, Achse ax *)
    w:=Real(CellWidth()); h:= Real(CellHeight());
    WITH CSi.DefPoints DO
      WITH Axis[ax] DO
        GetRotationMatrix(O,S, Matrix);
        Representation(Matrix, S.u, S.v, Surot, Svrot);
        du:= S.u-O.u; dv:= S.v-O.v;
        IF (dv>0.0) OR ((dv=0.0) AND (du>0.0)) THEN 
          Svrot:= Svrot-AxisTickLength;
          InverseRepr(Matrix, Surot, Svrot, startx, starty);
          IF dv<du THEN 
            startx:= startx-w/2.0;
            starty:= starty-yDistance-h;
          ELSIF (dv<=-du) AND (dv>0.0) THEN
            startx:= startx-w/2.0;
            starty:= starty+yDistance+yExtra;
          ELSIF dv>0.0 THEN
            startx:= startx+xDistance;
            starty:= starty-h/2.0;
          END; (*IF*)
        ELSE
          Svrot:= Svrot+AxisTickLength;
          InverseRepr(Matrix, Surot, Svrot, startx, starty);
          IF dv>=du THEN
            startx:= startx-w/2.0;
            starty:= starty-yDistance-h;
          ELSIF (-dv<=du) AND (dv<0.0) THEN
            startx:= startx-w/2.0;
            starty:= starty+yDistance+yExtra;
          ELSIF (dv<0.0) THEN
            startx:= startx+xDistance;
            starty:= starty-h/2.0;
          END; (*IF*)
        END; (*IF*)
        SetUCPen(startx,starty);
        CASE Log(units) OF
          -3: WriteString("0.001");
        | -2: WriteString("0.01");
        | -1: WriteString("0.1");
        |  0: Write("1");
        |  1: WriteString("10");
        |  2: WriteString("100");
        |  3: WriteString("1000");
        ELSE
          WriteString("1.0E"); WriteInt(Log(units),1);
        END; (*CASE*)
      END; (*WITH*)
    END; (*WITH*)
  END WriteTickLabel;
  
BEGIN (*DrawCS*)
  RemoveClipping; UCFrame; SetClipping(graphField);
  WITH CSi.DefPoints DO
    (* drawing of axis and scaling ticks, orthogonal to current axis,
         length: AxisTickLength: *)
    FOR ax:= X TO Z DO 
      WITH Axis[ax] DO SetUCPen(O.u,O.v); UCLineTo(F.u,F.v); AxisTick(S) END;
      WriteLabel(Axis[ax].F, CSi.AxisLabels[ax]);
      WriteTickLabel;
    END; (*FOR*)
  END; (*WITH*)
END DrawCS;


PROCEDURE Convert3DTo2D(P:Point3D; VAR xUC,yUC: REAL);
  (* Projects P on the two dimensional drawing plane. Returns user coordi-
     nates as defined by MacWindows.                                    *)
BEGIN (*Convert3DTo2D*)
  WITH CSi.DefVectors DO     (*corresponds to a matrix-multiplication*)
    xUC:= UnityVector[X].u * P.x +
          UnityVector[Y].u * P.y +
          UnityVector[Z].u * P.z + offsetVector.u;
    yUC:= UnityVector[X].v * P.x +
          UnityVector[Y].v * P.y +
          UnityVector[Z].v * P.z + offsetVector.v;
  END (*WITH*)
END Convert3DTo2D;

        
PROCEDURE Convert3DToPoint(P:Point3D; VAR x,y: INTEGER);
  (* Projects P on the two dimensional drawing plane. Returns pixel-window
     coordinates as defined by MacWindows.                              *)
  VAR xUC,yUC: REAL;
BEGIN (*Convert3DToPoint*)
  Convert3DTo2D(P,xUC,yUC); ConvertUCToPoint(xUC,yUC, x,y);
END Convert3DToPoint;
    

PROCEDURE ToggleEditPoints;
  CONST NotFilled = FALSE;
  VAR   x,y: INTEGER;
BEGIN (*ToggleEditPoints*)
  editPointsOn:= NOT editPointsOn;
  SetMode(invert); RemoveClipping;
  WITH CSi.DefPoints DO
    ConvertUCToPoint(O.u,O.v,x,y);
    Circle(x, y, radius, NotFilled, pat[dark]);
    FOR ax:= X TO Z DO 
      WITH Axis[ax] DO
        ConvertUCToPoint(F.u,F.v,x,y);
        Circle(x, y, radius, NotFilled, pat[dark]);
        ConvertUCToPoint(S.u,S.v,x,y);
        Circle(x, y, radius, NotFilled, pat[dark]);
      END; (*WITH*) 
    END; (*FOR*)
  END; (*WITH*)
  SetMode(paint); SetClipping(graphField);
END ToggleEditPoints;
     
  
PROCEDURE Select(P: Point2D); (*Highlights a selected point in black*)
  CONST Filled=TRUE; VAR x,y: INTEGER;
BEGIN
  ConvertUCToPoint(P.u,P.v,x,y);
  RemoveClipping; SetMode(invert);
     Circle(x, y, radius, Filled, pat[dark]);
  SetClipping(graphField); SetMode(paint);
END Select;


PROCEDURE DrawEditCS;
  (*deletes UCFrameContent, draws DefCS*)
      
  PROCEDURE DrawScaleChangePoints;
     VAR u,v, uRot, vRot, arrow: REAL; Matrix: Matrix2x2;
  BEGIN 
    WITH CSi DO
      FOR ax:= X TO Z DO
        arrow:= 2.0; (*size of arrow which indicates points where 
                       scaling changes.                                 *)
        WITH DefPoints DO
          WITH Axis[ax] DO
            GetRotationMatrix(O,F,Matrix);
            
            REPEAT
              u:= O.u + (F.u-O.u)/11.0;
              v:= O.v + (F.v-O.v)/11.0;
              Representation(Matrix, u,v, uRot, vRot);
              uRot:= uRot-arrow;
              InverseRepr(Matrix, uRot,vRot, u,v); SetUCPen(u,v);
              uRot:= uRot+arrow; vRot:= vRot+arrow;
              InverseRepr(Matrix, uRot,vRot, u,v); UCLineTo(u,v);
              vRot:= vRot-2.0*arrow;
              InverseRepr(Matrix, uRot,vRot, u,v); UCLineTo(u,v);
              uRot:= uRot-arrow; vRot:= vRot+arrow;
              InverseRepr(Matrix, uRot,vRot, u,v); UCLineTo(u,v);
              
              u:= O.u + (F.u-O.u)*10.0/11.0;
              v:= O.v + (F.v-O.v)*10.0/11.0;
              Representation(Matrix, u,v, uRot, vRot);
              uRot:= uRot+arrow;
              InverseRepr(Matrix, uRot,vRot, u,v); SetUCPen(u,v);
              uRot:= uRot-arrow; vRot:= vRot+arrow;
              InverseRepr(Matrix, uRot,vRot, u,v); UCLineTo(u,v);
              vRot:= vRot-2.0*arrow;
              InverseRepr(Matrix, uRot,vRot, u,v); UCLineTo(u,v);
              uRot:= uRot+arrow; vRot:= vRot+arrow;
              InverseRepr(Matrix, uRot,vRot, u,v); UCLineTo(u,v);
              
              arrow:= arrow-0.5;
            UNTIL arrow<=0.5;
          END; (*WITH*)
        END; (*WITH*)
      END; (*FOR*)
    END; (*WITH*)
  END DrawScaleChangePoints;
  

BEGIN (*DrawEditCS*)
  IF editPointsOn THEN ToggleEditPoints END;
  EraseUCFrameContent;
  d:=4; (*distance of axis labels from frame*)
    DrawCS;
  d:=2;
  DrawScaleChangePoints;
  ToggleEditPoints;  (*zeichnet die Kreise um die Punkte*)
END DrawEditCS;


PROCEDURE AllowedMousePosition(x,y: INTEGER):BOOLEAN;
  CONST AxisMoveZone = 40; AxisTickMoveZone = 24;
  VAR xr,yr,xUC,yUC: REAL; r1,r2: RectArea; xo,yo: INTEGER;
  
  PROCEDURE PointInRect(x,y: INTEGER; r: RectArea): BOOLEAN;
  BEGIN 
    RETURN  (x>=r.x) AND (x<=r.x+r.w) AND (y>=r.y) AND (y<=r.y+r.h);
  END PointInRect;
  
BEGIN (*AllowedMousePosition*)
  CASE CSedit OF
    AxisChange : 
      WITH graphField DO        (*calculation of outer rectangle:*)
        r1.x:= x - radius - 2;
        r1.y:= y - radius - 2;
        r1.w:= w + 2*radius + 4;
        r1.h:= h + 2*radius + 4;
      END; (*WITH*)
      WITH graphField DO        (*calculation of inner rectangle:*)
        r2.x:= x + AxisMoveZone;
        r2.y:= y + AxisMoveZone;
        r2.w:= w - 2*AxisMoveZone;
        r2.h:= h - 2*AxisMoveZone;
      END; (*WITH*)
      RETURN PointInRect(x,y,r1) AND NOT PointInRect(x,y,r2);
  | OriginChange:      
      WITH graphField DO        (*calculation of rectangle for origin:*)
        r1.x:= x + OriginProhibited;
        r1.y:= y + OriginProhibited;
        r1.w:= w - 2*OriginProhibited;
        r1.h:= h - 2*OriginProhibited;
      END; (*WITH*)
      RETURN PointInRect(x,y,r1);
  | ScaleChange:
      ConvertPointToUC(x,y,xUC,yUC);
      Representation(RotationMatrix, xUC,yUC, xr,yr);
      WITH r1 DO
        x:= ouint;
        y:= ovint-AxisTickMoveZone;
        w:= fuint-ouint;
        h:= 2*AxisTickMoveZone;
      END; (*WITH*)
      ConvertUCToPoint(xr,yr, xo,yo);(*rotated mouse coord. in wind.pixels*)
      RETURN PointInRect(xo,yo,r1);
  END; (*CASE*)
END AllowedMousePosition;


PROCEDURE DuringDrag(mouseX, mouseY: INTEGER);
  VAR mouseU, mouseV, u, v: REAL; curC, saveC: Color;
  
  PROCEDURE DrawAxisOutline;
    VAR originX, originY, test1, test2, fac, x, y: INTEGER; 
  BEGIN (*DrawAxisOutline*)
    ConvertUCToPoint(CSi.DefPoints.O.u,CSi.DefPoints.O.v,originX,originY);
    test1:= ABS(originX-mouseX);
    test2:= ABS(originY-mouseY);
    IF test1<test2 THEN test1:= test2 END; (* test1:=Max(test1,test2) *)
    test2:= graphField.w + graphField.h;(*estimate for length of diagonal*)
    IF test1=0 THEN fac:=0 ELSE fac:= (test2 DIV test1) + 2 END;
    x:= originX + fac*(mouseX-originX);
    y:= originY + fac*(mouseY-originY);
    SetPen(originX, originY); LineTo(x,y);
  END DrawAxisOutline;
  
  PROCEDURE DrawOriginOutline;
    VAR originX, originY, x, y, test1, test2, d, fac: INTEGER;
  BEGIN (*DrawOriginOutline*)
    d:= graphField.w + graphField.h; (*estimate for length of diagonal*)
    WITH CSi.DefPoints DO 
      ConvertUCToPoint(O.u, O.v, originX, originY);
      FOR ax:= X TO Z DO
        ConvertUCToPoint(Axis[ax].F.u, Axis[ax].F.v, x, y);
        x:= x-originX;
        y:= y-originY;
        test1:= ABS(x); test2:= ABS(y);
        IF test1<test2 THEN test1:= test2 END; (*test1:= Max(test1,test2)*)
        IF test1=0 THEN test1:=1 END;
        test2:= graphField.w + graphField.h; (*estimate f.length of diag.*)
        fac:= (test2 DIV test1) + 2;
        x:= fac * x;
        y:= fac * y;
        SetPen(mouseX,mouseY); LineTo(mouseX+x,mouseY+y);
      END; (*FOR*)
    END; (*WITH*)
  END DrawOriginOutline;
  
  PROCEDURE DrawTickOutline;
    VAR xStart,xStop,yStart,yStop,x,y,mouseUrot,mouseVrot: REAL; 
  BEGIN (*DrawTickOutline*)
    ConvertPointToUC(mouseX, mouseY, mouseU, mouseV);
    Representation(RotationMatrix,mouseU,mouseV,mouseUrot,mouseVrot);
    xStart:= mouseUrot;
    xStop := mouseUrot;
    yStart:= ov - AxisTickLength;
    yStop := ov + AxisTickLength;
    InverseRepr(RotationMatrix,xStart,yStart, x,y);
    SetUCPen(x,y);
    InverseRepr(RotationMatrix,xStop,yStop, x,y);
    UCLineTo(x,y);
  END DrawTickOutline;

BEGIN (*DuringDrag*)
  GetColor(saveC);
  curC:= saveC; curC.greyContent:= grey;
  SetColor(curC); SetMode(invert);
  IF AllowedMousePosition(mouseX,mouseY) THEN
    CASE CSedit OF
      AxisChange :  DrawAxisOutline;
    | OriginChange: DrawOriginOutline;
    | ScaleChange:  DrawTickOutline;
    END; (*CASE*)
  END; (*IF*)
  SetColor(saveC); SetMode(paint);
END DuringDrag;

PROCEDURE AfterDrag(mouseX,mouseY: INTEGER);
  VAR PreDefPoints: CSDefPointsType; (*Preliminary Definitions Points*)
      u,v,tu,tv, (*t ranslation u, v*) 
      mouseU, mouseV,mouseUrot,mouseVrot, uRot,vRot: REAL; 
      
  PROCEDURE Distance(P1,P2:Point2D): REAL; 
    (*returns distance of the given points in UC*)
    VAR du,dv,distance: REAL;
  BEGIN
    du:=P1.u-P2.u; dv:= P1.v-P2.v;
    distance:= Sqrt((du*du)+(dv*dv));
    RETURN distance;
  END Distance;
  
  PROCEDURE IntersectionWithFrame(u1,v1,u2,v2:REAL; VAR u,v: REAL);
  (*This procedure intersects the straight line starting from P1 and 
    passing by P2 with the frame CSi.Frame. It returns the 
    coordinates of the intersection point in u and v                    *)
  TYPE DirType = (upRight, upLeft, downLeft, downRight);
  VAR  dir: DirType; du, slope: REAL;
    PROCEDURE Intersect;
      PROCEDURE Fu(vConst:REAL; VAR u:REAL);
      BEGIN v:=vConst; u:=(v-v1)/slope+u1 END Fu;
      
      PROCEDURE Fv(uConst:REAL; VAR v:REAL);
      BEGIN u:=uConst; v:=(u-u1)*slope+v1 END Fv;
    BEGIN (*Intersect*)
      WITH CSi.Frame DO
        CASE dir OF
          upRight  : Fv(uMax,v); IF v>vMax THEN Fu(vMax,u) END;
        | upLeft   : Fu(vMax,u); IF u<uMin THEN Fv(uMin,v) END;
        | downLeft : Fv(uMin,v); IF v<vMin THEN Fu(vMin,u) END;
        | downRight: Fu(vMin,u); IF u>uMax THEN Fv(uMax,v) END;
        END; (*CASE*)
      END; (*WITH*)
    END Intersect;
    
  BEGIN (*IntersectionWithFrame*)
    IF v2>=v1 THEN
      IF u2>=u1 THEN dir:= upRight ELSE dir:= upLeft END;
    ELSE
      IF u2>=u1 THEN dir:= downRight ELSE dir:= downLeft END;
    END; (*IF*)
    du:= u2-u1;
    IF du=0.0 THEN
      CASE dir OF 
        upRight, upLeft    : u:= u1; v:= CSi.Frame.vMax;
      | downLeft, downRight: u:= u1; v:= CSi.Frame.vMin;
      END; (*CASE*)
    ELSE
      slope:= (v2-v1)/du;
      IF slope=0.0 THEN
        CASE dir OF 
          upRight, downRight: u:= CSi.Frame.uMax; v:= v1;
        | upLeft, downLeft  : u:= CSi.Frame.uMin; v:= v1;
        END; (*CASE*)
      ELSE
        Intersect;
      END; (*IF*)
    END; (*IF*)
  END IntersectionWithFrame;

  PROCEDURE UpdateScalingPoints; 
    (*Updates Scaling Points after translation of origin*)
    VAR fac: REAL;
  BEGIN (*UpdateScalingPoints*)
    FOR ax:= X TO Z DO
      fac:= Distance(PreDefPoints.O, PreDefPoints.Axis[ax].F)/
            Distance(CSi.DefPoints.O, CSi.DefPoints.Axis[ax].F);
      WITH PreDefPoints DO
        Axis[ax].maxUnits:= CSi.DefPoints.Axis[ax].maxUnits * fac;
      END; (*WITH*)
    END; (*FOR*)
    CSi.DefPoints:= PreDefPoints;
    FOR ax:= X TO Z DO 
      WITH CSi.DefPoints.Axis[ax] DO units:= GetUnits(maxUnits) END;
      InstallScalepoint(ax);
    END;
  END UpdateScalingPoints;

PROCEDURE CalculateNewMaxUnits(a: AxisIdentifier);  
  (*Given the provisional scalepoint S and its units, this procedure
    calculates maxunits. With that value, the procedure GetUnits updates
    maxUnits, thereafter InstallScalePoint calculates the definitive 
    scalepoint                                                          *)
  VAR distF,distS: REAL;
BEGIN 
  WITH CSi.DefPoints DO WITH Axis[a] DO
    distF:= Distance(F, O);
    distS:= Distance(S, O);
    maxUnits:= units * (distF/distS);
  END END; (*WITH*)
END CalculateNewMaxUnits;
  
BEGIN (*AfterDrag*)
  ToggleEditPoints;
  CASE CSedit OF
    AxisChange :
        Select(CSi.DefPoints.Axis[a].F);
        IF AllowedMousePosition(mouseX,mouseY) THEN
          ConvertPointToUC(mouseX,mouseY,u,v);
          WITH CSi.DefPoints DO
            IntersectionWithFrame(O.u,O.v,u,v,Axis[a].F.u,Axis[a].F.v);
            (*  DecideWetherNewPointAllowed(PreDefPoints.A): This 
                procedure could e.g. interrupt new definition if 
                axis to short, two axis at the same place and so on.    *)
            InstallScalepoint(a);
          END; (*WITH*)
        END; (*IF*)
           
  | OriginChange: 
        Select(CSi.DefPoints.O);
        PreDefPoints:= CSi.DefPoints;
        IF AllowedMousePosition(mouseX,mouseY) THEN
              ConvertPointToUC(mouseX, mouseY, u, v);
              PreDefPoints.O.u:= u;
              PreDefPoints.O.v:= v;
              tu:= u - CSi.DefPoints.O.u;
              tv:= v - CSi.DefPoints.O.v;
              WITH PreDefPoints DO
                FOR ax:= X TO Z DO
                  IntersectionWithFrame(O.u,O.v,
                   CSi.DefPoints.Axis[ax].F.u+tu, CSi.DefPoints.Axis[ax].F.v+tv,
                   Axis[ax].F.u,                   Axis[ax].F.v);
                END; (*FOR*)
              END; (*WITH*)
              UpdateScalingPoints;
        END; (*IF*)
            
  | ScaleChange: 
        Select(CSi.DefPoints.Axis[a].S);
        IF AllowedMousePosition(mouseX,mouseY) THEN
            ConvertPointToUC(mouseX, mouseY, mouseU, mouseV);
            Representation
                 (RotationMatrix,mouseU,mouseV,mouseUrot,mouseVrot);
            uRot:= mouseUrot;
            vRot:= ov;
            WITH CSi.DefPoints.Axis[a].S DO
              InverseRepr(RotationMatrix,uRot,vRot, u,v);
            END; (*WITH*)
            CalculateNewMaxUnits(a);
            WITH CSi.DefPoints.Axis[a] DO units:= GetUnits(maxUnits) END;
            InstallScalepoint(a);
        END; (*IF*)
  END; (*CASE*)
  DrawEditCS;
END AfterDrag;


PROCEDURE ExitP; (*This procedure is called by exit menu*)
BEGIN
  exit:= TRUE;
END ExitP;


PROCEDURE EditCS;
  VAR i,j: INTEGER;  
      dummy : ClickKind; 
      mt,mCmdt: ARRAY [0..64] OF CHAR;
  
  PROCEDURE Fu(P: Point2D): INTEGER; (* returns integer x-coord. of P *)
    VAR x,y: INTEGER;
  BEGIN ConvertUCToPoint(P.u,P.v,x,y); RETURN x END Fu;
 
  PROCEDURE Fv(P: Point2D): INTEGER; (* returns integer y-coord. of P *)
    VAR x,y: INTEGER;
  BEGIN ConvertUCToPoint(P.u,P.v,x,y); RETURN y END Fv;
    
  PROCEDURE UpdateCS(CS: CSType);
  (*This procedure is used to update "currentCS" after editing*)
  BEGIN
    WITH CSi.DefPoints DO
      WITH CS DO
        WITH origin DO ConvertUCToPoint(O.u,O.v, x,y) END;
        WITH xPoint DO WITH Axis[X] DO 
          ConvertUCToPoint(F.u,F.v, x,y); xMax:= maxUnits; 
        END END (*WITH WITH*);
        WITH yPoint DO WITH Axis[Y] DO 
          ConvertUCToPoint(F.u,F.v, x,y); yMax:= maxUnits; 
        END END (*WITH WITH*);
        WITH zPoint DO WITH Axis[Z] DO 
          ConvertUCToPoint(F.u,F.v, x,y); zMax:= maxUnits; 
        END END (*WITH WITH*);
      END; (*WITH*)
    END; (*WITH*)
  END UpdateCS;
     
BEGIN (*EditCS*)
  DisableDeskAccessories;
  GetString(curMText,mt);
  GetString(curMCmdText,mCmdt);
  InstallMenu(ExitMenu, mt, enabled);
  InstallCommand(ExitMenu, exitPos, mCmdt, ExitP, enabled, unchecked);
  InstallAliasChar(ExitMenu, exitPos,curAliasChar);
  UseMenu(ExitMenu);
  DrawEditCS;
  (*deletes previous mouse clicks: *) GetLastMouseClick(i,j,dummy);
  exit:= FALSE;
  REPEAT
    DialogMachineTask;(*dieser Aufruf faellt weg, sobald dies die neue
      Version von DMWindows in PointClicked uebernimmt.                *)
    FOR a:= X TO Z DO   
      IF PointClicked           (*one of the axis selected*)
        (Fu(CSi.DefPoints.Axis[a].F), Fv(CSi.DefPoints.Axis[a].F),radius) 
      THEN
        CSedit:= AxisChange; 
        Select(CSi.DefPoints.Axis[a].F);
        Drag(DuringDrag, AfterDrag);
      ELSIF 
        PointClicked            (*scaling of a axis selected*)
          (Fu(CSi.DefPoints.Axis[a].S),Fv(CSi.DefPoints.Axis[a].S),radius) 
      THEN
        (*That rotation is needed during editing of axistick:*)
        WITH CSi.DefPoints DO
          WITH Axis[a] DO
            GetRotationMatrix(O,F,RotationMatrix);
            Representation(RotationMatrix, F.u,F.v, fu,fv);
            ConvertUCToPoint(fu, fv, fuint, fvint);
            Representation(RotationMatrix, O.u,O.v, ou,ov);
            ConvertUCToPoint(ou, ov, ouint, ovint);
          END; (*WITH*)
        END; (*WITH*)
        CSedit:= ScaleChange;
        Select(CSi.DefPoints.Axis[a].S);
        Drag(DuringDrag, AfterDrag);
      END; (*IF*)
    END; (*FOR*)
    IF PointClicked             (*origin selected*)
      (Fu(CSi.DefPoints.O),Fv(CSi.DefPoints.O),radius) THEN     
        CSedit:= OriginChange;
        Select(CSi.DefPoints.O);
        Drag(DuringDrag, AfterDrag)
    END; (*IF*)
  UNTIL exit;
  
  DefPointsToDefVectors;
  UpdateCS(currentCS);
  ToggleEditPoints;      (*2nd call of this procedure deletes highlight
                           circles and draws normal CS                  *)
    (*These calls should prevent from update events:*)
  DialogMachineTask;DialogMachineTask;DialogMachineTask;
  RemoveMenu(ExitMenu);
  EraseUCFrameContent;
  EnableDeskAccessories;
END EditCS;



PROCEDURE SetTermMenuText(mText,mCmdText: ARRAY OF CHAR; 
						  aliasChar: CHAR);
BEGIN
  IF curMText = VAL(String,NIL) (* implies curMCmdText=NIL *) THEN
    curMText:= NewString(mText);
    curMCmdText:= NewString(mCmdText);
  ELSE
    PutString(curMText,mText);
    PutString(curMCmdText,mCmdText);
  END(*IF*);
  curAliasChar:= aliasChar;
END SetTermMenuText;

PROCEDURE GetTermMenuText(VAR mText,mCmdText: ARRAY OF CHAR; 
						  VAR aliasChar: CHAR);
BEGIN
  GetString(curMText,mText);
  GetString(curMCmdText,mCmdText);
  aliasChar:= curAliasChar;
END GetTermMenuText;


  VAR 
    loadLev: CARDINAL;
    installationOk: BOOLEAN;

  PROCEDURE TermGraphics3D ;
  BEGIN
    IF CurrentDMLevel()=loadLev THEN
      DisposeString(curMText);
      DisposeString(curMCmdText);
    END;(*IF*)
  END TermGraphics3D ;



BEGIN
  loadLev:=CurrentDMLevel();
  editPointsOn:= FALSE;
  d:= 2;
  curMText:= notAllocatedString;
  curMCmdText:= notAllocatedString;
  CASE CurrentLanguage() OF
  German : SetTermMenuText("KS-Editor","Neudefinition beenden",0C);
  ELSE     SetTermMenuText("CS-Editor","Terminate CS-editing",0C);
  END; (*CASE*)
  InstallTermProc( TermGraphics3D,installationOk );
END Graphics3D.