IMPLEMENTATION MODULE PolyLib;
  (******************************************************************)
  (*                                                                *)
  (*        MODULE: PolyLib,              Version   0.4 / 003       *)
  (*                                                                *)
  (*        Purpose:                                                *)
  (*                                                                *)
  (*        Implementation:  For Apple Macintosh                    *)
  (*                         Version fr  MacMETH V2.0              *)
  (*                         based on DM 0.4                        *)
  (*                                                                *)
  (*        Programming:     A.Itten                                *)
  (*                         C E L T I A                            *)
  (*                         ETH-Zentrum                            *)
  (*                         CH-8092 Zrich                         *)
  (*                                                                *)
  (*        LAST MODIFICATION :  25.11.86                           *)
  (******************************************************************)
  FROM ComplexLib IMPORT Complex,
                         one, zero,imagUnit,
                         SetCom, GetCom,
                         PolToKart, KartToPol,
                         MultCom, DivCom, AddCom, SubCom, NegCom,
                         ExpCom, SkalCom, SetToZero;  
  
  CONST minNonZero    = 1.0E-5;          (* min denominator *)

  PROCEDURE InitPolynom(VAR P:Polynom; name: NamenString; n: CARDINAL);
  VAR i    : CARDINAL;
  BEGIN
    P.name:= name;
    P.curOrder:= n;
    FOR i:= 0 TO maxPolySize DO
      SetToZero(P.koef[i]);
    END; (* FOR *)
  END InitPolynom;

  PROCEDURE SetPolynom(VAR p: Polynom; coef: ARRAY OF Complex);
    VAR i,n : INTEGER;
  BEGIN
    n:= HIGH(coef);
    IF n > maxPolySize THEN n:= maxPolySize; END;
    FOR i:= 0 TO n DO p.koef[i]:= coef[i]; END;
    p.curOrder:= n;
  END SetPolynom;

  PROCEDURE FormPolynom(VAR p: Polynom; roots: ARRAY OF Complex);
    VAR i,n : INTEGER;
  
    PROCEDURE Ausmult(VAR p: Polynom; pol: Complex);
    VAR j: CARDINAL;
    BEGIN
      WITH p DO
        INC(curOrder);
        koef[curOrder]:= koef[curOrder-1];
        FOR j:= p.curOrder - 1 TO 1 BY -1 DO
          MultCom(koef[j],pol,koef[j]);
          AddCom(koef[j],koef[j-1],koef[j]);
        END;  (* FOR *)
        MultCom(koef[0],pol,koef[0]);
      END; (* WITH *)
    END Ausmult;

  BEGIN
    InitPolynom(p,"NoName",0);
    n:= HIGH(roots);
    IF n > maxPolySize THEN n:= maxPolySize END;
    FOR i:= 0 TO n DO 
      IF p.curOrder = 0 THEN
        NegCom(roots[0]); 
        p.koef[0]:= roots[0];
        SetCom(p.koef[1],1.0,0.0);
        p.curOrder:= 1;
      ELSE
        NegCom(roots[i]);
        Ausmult(p,roots[i]);
      END; (* IF *) 
    END; (* FOR *)
  END FormPolynom;

  PROCEDURE FindOneRoot(VAR P:Polynom;
                        VAR z: Complex;
                        maxError: REAL; 
                        maxIteration : CARDINAL;
                        VAR  done: BOOLEAN);
  
  VAR i,j             : CARDINAL;
      b, c, korr      : Complex;
      betr, phi       : REAL;
      turnNumber      : CARDINAL;
      redPol          : Polynom;
                     
  BEGIN
    j:=0;          (* Schleifenzhler *)
    SetToZero(b); SetToZero(korr);
    InitPolynom(redPol,'reduziertes Polynom',P.curOrder-1);
    turnNumber:= maxIteration DIV 2;
   
    REPEAT
      INC(j);     (* Schleifenzhler *)
      b:= P.koef[P.curOrder];
      redPol.koef[P.curOrder-1]:= b;

      IF j = turnNumber THEN  
        (* wenn die Iteration schlecht konvergiert, dann wird korr um 90 
           gedreht und etwas dazu addiert im Fall z = 0.0 ist *)
        MultCom(z,imagUnit,z);
        SetCom(c,0.1,0.1);
        AddCom(z,c,z);
      END;  (* IF *)

      SetToZero(c);
   
      FOR i:=(P.curOrder-1) TO 0 BY -1 DO
        MultCom(c,z,c); AddCom(c,b,c);
        MultCom(b,z,b); AddCom(b,P.koef[i],b);
        IF i>0 THEN redPol.koef[i-1]:= b END;
      END;  (* FOR *)
      
      KartToPol(c,betr,phi);
      IF (betr < minNonZero) THEN   (* no div by zero *)
        SetCom(korr,1.0,-1.0);  (* probably not best sollution *)
      ELSE 
        DivCom(b,c,korr);
      END;  (* IF *)
      SubCom(z,korr,z); 
      
      KartToPol(b,betr,phi);
         
    UNTIL (betr < maxError) OR (j > maxIteration) ; 
      (* = maximaler Fehler *)
    IF (j <= maxIteration) THEN 
      done := TRUE;
      P:= redPol;
    ELSE  
      done := FALSE;
    END;  (* IF *) 
  END FindOneRoot;

  PROCEDURE FindRoots(P:Polynom;
                      VAR roots: ARRAY OF Complex;
                      zInit: Complex;
                      maxError: REAL; 
                      maxIteration : CARDINAL;
                      VAR  found: CARDINAL;
                      VAR allDone: BOOLEAN);
                      
  VAR i,n       : CARDINAL;
      conjCompl : BOOLEAN;
      re,im     : REAL;  
      done      : BOOLEAN; 
                      

  BEGIN
    conjCompl:= FALSE; found:= 0; allDone := TRUE;
    roots[0]:= zInit;
    n:= HIGH(roots); 
    IF n > P.curOrder THEN n:= P.curOrder; END;
    FOR i:= 0 TO n DO
      FindOneRoot(P, roots[i], maxError, maxIteration, done);      
      IF done = TRUE THEN
        INC(found);
        GetCom(roots[i],re,im);
        IF ((im < -minNonZero) OR (im>minNonZero)) AND NOT conjCompl THEN
          IF i < n THEN SetCom(roots[i+1], re,-im); END;
          conjCompl:= TRUE;
        ELSE
          (* IF i < n THEN roots[i+1]:= zInit; END; *)
          conjCompl:= FALSE;
        END; (* IF *)
      ELSE 
        allDone:= FALSE;
      END; (* IF *)
    END; (* FOR *)
  END FindRoots;
  

BEGIN  (* PolyLib *)
END PolyLib.
