IMPLEMENTATION MODULE Protocolling ;

  (*

        Implementation and Revisions:
        ============================

        Author  Date        Description
        ------  ----        -----------

        af      03/09/90    First implementation (DM 2.0,
                            MacMETH 2.6+)
        dg      09/01/92    Imports from DMClock instead DateAndTime
        					Import from Scanner ommitted.

  *)
         
  FROM DMFiles   IMPORT TextFile;
  FROM DMWindows IMPORT NoBackground, Window, notExistingWindow,
                        WindowKind, ScrollBars, CloseAttr, ZoomAttr,
                        WindowFrame, CreateWindow, WFFixPoint,
                        WindowExists, AutoRestoreProc,
                        RedrawBackground, PutOnTop;
                              
  FROM DMWindowIO IMPORT SelectForOutput, EraseContent,
  						 BackgroundWidth, BackgroundHeight,
                         SetPos,
                         AssignWindowFont, WindowFont,
                         WriteString, Write, WriteLn, 
                         WriteCard, WriteInt;

  FROM DMStrings IMPORT Length, AssignString;
  
  FROM DMClock   IMPORT InterpreteSeconds, NowInSeconds;
  FROM WriteDatTim IMPORT WriteDate, DateFormat, WriteTime, TimeFormat,
                          DateAndTimeRec;
  
  (* * * * * * 
    This module has been used for the implementation of a cross-
    reference listing generator. It reported reading an input 
    file and writing to an output file.

    FROM Scanner IMPORT inFile, outFile;
    
    Below, the corresponding variables are defined allowing  
    correct compilation.
  * * * * * * *)
  
  VAR
    inFile, outFile: TextFile;

  VAR
    wp:    Window;

    
   PROCEDURE Pos(subString, string: ARRAY OF CHAR): INTEGER;
     (*--------------------------------------------------------------*)
     (* PROCEDURE Pos -- find position of the first occurence of     *)
     (* 'subString' in 'string'. HIGH(string) + 1 is returned if no  *)
     (* match was detected or 'subString' was empty.                 *)
     (*                                                              *)
     (* Rewritten by A.Fritschy, Inst. f. Phytomedizin, ETH Zrich,  *)
     (* Apr-10-1986                                                  *) 
     (*--------------------------------------------------------------*)
   VAR
     searched, ch: CHAR;
     i,j,lasti,len   : INTEGER;
   BEGIN
     (* calculate lengths once only - increaes speed   *)
     len:=Length(subString);
     lasti:=Length(string)-len;
  
     IF len > 0 THEN                  (* 'string' start search index *)
       i:=0;
       WHILE i<=lasti DO
         j:=0;                        (* 'subString' index           *)
         REPEAT
           searched:=subString[j];
           ch      :=string[i+j];
           INC(j);
         UNTIL (ch # searched) OR (j=len);
         IF (ch # searched) THEN      (* start with next position    *)
           INC(i);
         ELSE                         (* position found              *)
           RETURN i
         END (* IF *);
       END (* WHILE *);
     END (* IF *);
     RETURN HIGH(string)+1;           (* if position not found       *)
   END Pos;


  PROCEDURE ActivateProtocolWindow;
    VAR wpf:   WindowFrame;
  BEGIN
    IF NOT WindowExists( wp ) THEN
      wpf.w:=BackgroundWidth()-10; wpf.h:=BackgroundHeight()-10;
      wpf.x:= (BackgroundWidth()-wpf.w) DIV 2;
      wpf.y:= (BackgroundHeight()-wpf.h) DIV 2; 
      CreateWindow( wp, FixedSize, WithoutScrollBars,
                    WithCloseBox, WithoutZoomBox, bottomLeft, wpf, 'Protocol',
                    AutoRestoreProc );
    ELSE
      PutOnTop( wp );
    END; (* IF *)
    SelectForOutput(wp);
    AssignWindowFont( Monaco, 9 );
    EraseContent();
    SetPos(1,1);
  END ActivateProtocolWindow;
   
  PROCEDURE StartProtocol;
    VAR cl: INTEGER; dtr: DateAndTimeRec; df: DateFormat;
  BEGIN
    SelectForOutput(wp);
    ActivateProtocolWindow;

    cl := 2;
    SetPos( cl,  2 ); WriteString( "Reading from file: '" );
    WriteString( inFile.filename ); WriteString("'   ");
    INC(cl);
    IF ( 
         ( Pos( ".MOD", inFile.filename ) >= Length( inFile.filename ) ) &
         ( Pos( ".DEF", inFile.filename ) >= Length( inFile.filename ) ) &
         ( Pos( ".PAS", inFile.filename ) >= Length( inFile.filename ) ) &
         ( Pos( ".P",   inFile.filename ) >= Length( inFile.filename ) ) &
         ( Pos( ".F",   inFile.filename ) >= Length( inFile.filename ) )  
       ) 
    THEN
       SetPos( cl,  2 );
       WriteString( "--> Warning: Extension in name of processed file suggests that it is not"); 
       INC(cl);
       SetPos( cl,  2 );
       WriteString( "    a Pascal, Modula-2 or FORTRAN source-file?"); INC(cl);
    END; (* IF *)
    SetPos( cl,  2 ); WriteString( "Writing to file: '" );
    WriteString( outFile.filename ); Write("'"); INC(cl);
    SetPos( cl,  2 );
    WriteString("Starting processing on ");
    WITH dtr DO
      InterpreteSeconds( NowInSeconds(), year, month, day, hour, minute, second, dayOfWeek );
    END;(*WITH*)
    WriteDate(dtr,Write,letMonth); WriteString(" at ");
    WriteTime(dtr,Write,brief24hSecs);
    INC(cl);

    SetPos( dispProgressAtLn, dispProgressAtCol+5); 
    WriteString( "lines processed." );
  END StartProtocol;

  PROCEDURE EraseProtocol;
  BEGIN
    SelectForOutput(wp);
    EraseContent();
    SetPos(1,1);
  END EraseProtocol;

  PROCEDURE ReportString (s: ARRAY OF CHAR);
  BEGIN
    SelectForOutput(wp); WriteString(s);
  END ReportString;

  PROCEDURE ReportLine (s: ARRAY OF CHAR);
  BEGIN
    SelectForOutput(wp);  WriteString(s); WriteLn;
  END ReportLine;
  
  PROCEDURE ReportIntAt (inRow,atCol, i, le: INTEGER);
  BEGIN
    SetPos(inRow,atCol); WriteInt(i,le);
  END ReportIntAt;
  
  PROCEDURE TerminateProtocol;
    VAR dtr: DateAndTimeRec;
  BEGIN
    SelectForOutput(wp);
    SetPos( dispProgressAtLn-1, dispProgressAtCol); 
    WriteString( "Processing finished on " );
    WITH dtr DO
      InterpreteSeconds( NowInSeconds(), year, month, day, hour, minute, second, dayOfWeek );
    END;(*WITH*)
    WriteDate(dtr,Write,letMonth); WriteString(" at ");
    WriteTime(dtr,Write,brief24hSecs);
  END TerminateProtocol;


BEGIN
  wp := notExistingWindow;
  AssignString('(unknown input file)',inFile.filename);
  AssignString('(unknown output file)',outFile.filename);
END Protocolling .
