IMPLEMENTATION MODULE Splines;

  (*

        Implementation and Revisions:
        ============================

        Author  Date        Description
        ------  ----        -----------

        or      29.06.90    First implementation (DM 1.1, MacMETH 2.6)
        or      29.06.90    Bug fixed
        dg  	09/01/91    Now imports from DMMathLib 

  *)

  FROM DMStorage  IMPORT Allocate, Deallocate;
  FROM DMWindowIO IMPORT SetUCPen, UCDot, UCLineTo, SetPen, LineTo;
  FROM DMMathLib  IMPORT Sqrt;



  PROCEDURE Sqr( x: REAL): REAL;
  BEGIN
    RETURN x*x;
  END Sqr;


  PROCEDURE Spline( n: INTEGER;  VAR x, y, y1, y2: ARRAY OF REAL );
    VAR
      k: INTEGER;
      h, a: POINTER TO ARRAY[0..8000] OF REAL;
  BEGIN
    Allocate( h, SIZE(REAL) *(n+1) );
    Allocate( a, SIZE(REAL) *(n+1) );
    FOR k:= 0 TO n-1 DO
      h^[k]:= x[k+1] - x[k];
    END(*FOR*);
    FOR k:= 1 TO n-1 DO
      a^[k]:= 2.0*(h^[k-1] + h^[k]);
    END(*FOR*);
    FOR k:= 1 TO n-1 DO
      y2[k]:= 6.0 *((y[k+1]-y[k])/h^[k] -(y[k]-y[k-1])/h^[k-1]);
    END(*FOR*);
    y2[0]:= 0.0;
    y2[n]:= 0.0;
    FOR k:= 2 TO n-1 DO
      a^[k]:= a^[k] -Sqr(h^[k-1])/a^[k-1];
    END(*FOR*);
    FOR k:= 2 TO n-1 DO
      y2[k]:= y2[k] -y2[k-1]*h^[k-1]/a^[k-1];
    END(*FOR*);
    FOR k:= n-1 TO 1 BY -1 DO
      y2[k]:= (y2[k]-y2[k+1]*h^[k])/a^[k];
    END(*FOR*);
    FOR k:= 0 TO n-1 DO
      y1[k]:= (y[k+1]-y[k])/h^[k] -(y2[k+1]+2.0*y2[k])*h^[k]/6.0;
    END(*FOR*);
    y1[n]:= y1[n-1]+h^[n-1]*y2[n-1]/2.0;
    Deallocate( h );
    Deallocate( a );
  END Spline;


  PROCEDURE Intrpl( x0: REAL;  n: INTEGER;
                    VAR x, y, y1, y2: ARRAY OF REAL ): REAL;
    VAR
      a, b, m: INTEGER;
      h : REAL;
  BEGIN
    a:= 0;
    b:= n-1;
    WHILE a < b DO
      m:= (a+b+1) DIV 2;
      IF (x[m] <= x0) THEN  a:= m;
      ELSE                  b:= m-1;
      END(*IF*);
    END(*WHILE*);
    h:= x0-x[a];
    RETURN y[a] +h*(y1[a]
                +h*(y2[a]/2.0
                +h*((y2[a+1]-y2[a])/(6.0*x[a+1]-x[a]))));
  END Intrpl;


  PROCEDURE Spline1( n: INTEGER;  ya, yb: REAL;
                     VAR x, y, y1, y2: ARRAY OF REAL );
    VAR
      k: INTEGER;
      h, a: POINTER TO ARRAY[0..8000] OF REAL;
  BEGIN
    Allocate( h, SIZE(REAL) *(n+1) );
    Allocate( a, SIZE(REAL) *(n+1) );
    FOR k:= 0 TO n-1 DO
      h^[k]:= x[k+1] - x[k];
    END(*FOR*);
    a^[0]:= 2.0*h^[0];
    a^[n]:= 2.0*h^[n-1];
    FOR k:= 1 TO n-1 DO
      a^[k]:= 2.0*(h^[k-1] + h^[k]);
    END(*FOR*);
    y2[0]:= 6.0 *((y[1]-y[0])/h^[0] -ya);
    y2[n]:= 6.0 *(yb -(y[n]-y[n-1])/h^[n-1]);
    FOR k:= 1 TO n-1 DO
      y2[k]:= 6.0 *((y[k+1]-y[k])/h^[k] -(y[k]-y[k-1])/h^[k-1]);
    END(*FOR*);
    FOR k:= 1 TO n-1 DO
      a^[k]:= a^[k] -Sqr(h^[k-1])/a^[k-1];
    END(*FOR*);
    FOR k:= 1 TO n-1 DO
      y2[k]:= y2[k] -y2[k-1]*h^[k-1]/a^[k-1];
    END(*FOR*);
    y2[n]:= y2[n]/a^[n];
    FOR k:= n-1 TO 1 BY -1 DO
      y2[k]:= (y2[k]-y2[k+1]*h^[k])/a^[k];
    END(*FOR*);
    FOR k:= 1 TO n-1 DO
      y1[k]:= (y[k+1]-y[k])/h^[k] -(y2[k+1]+2.0*y2[k])*h^[k]/6.0;
    END(*FOR*);
    y1[0]:= ya;
    y1[n]:= yb;
    Deallocate( h );
    Deallocate( a );
  END Spline1;


  PROCEDURE Spline2( n: INTEGER;  VAR x, y, y1, y2: ARRAY OF REAL );
  (* n >= 3, y[n] = y[0] *)
    VAR
      k: INTEGER;
      h, a, c: POINTER TO ARRAY[0..8000] OF REAL;
  BEGIN
    Allocate( h, SIZE(REAL) *(n+1) );
    Allocate( a, SIZE(REAL) *(n+1) );
    Allocate( c, SIZE(REAL) *(n+1) );
    FOR k:= 0 TO n-1 DO
      h^[k]:= x[k+1] - x[k];
    END(*FOR*);
    a^[0]:= 2.0*(h^[n-1]+h^[0]);
    FOR k:= 1 TO n-1 DO
      a^[k]:= 2.0 * (h^[k-1] + h^[k]);
    END(*FOR*);
    y2[0]:= 6.0 *((y[1]-y[0])/h^[0] -(y[n]-y[n-1])/h^[n-1]);
    FOR k:= 1 TO n-1 DO
      y2[k]:= 6.0 *((y[k+1]-y[k])/h^[k] -(y[k]-y[k-1])/h^[k-1]);
    END(*FOR*);
    c^[0]:= h^[n-1];
    FOR k:= 1 TO n-3 DO
      a^[k]:= a^[k] -Sqr(h^[k-1])/a^[k-1];
      c^[k]:= -h^[k-1]*c^[k-1]/a^[k-1];
    END(*FOR*);
    a^[n-2]:= a^[n-2]-Sqr(h^[n-3])/a^[n-3];
    c^[n-2]:= h^[n-2]-h^[n-3]*c^[n-3]/a^[n-3];
    FOR k:= 0 TO n-2 DO
      a^[n-1]:= a^[n-1]-Sqr(c^[k])/a^[k];
    END(*FOR*);
    FOR k:= 1 TO n-2 DO
      y2[k]:= y2[k] -y2[k-1]*h^[k-1]/a^[k-1];
    END(*FOR*);
    FOR k:= 0 TO n-2 DO
      y2[n-1]:= y2[n-1] -y2[k]*c^[k]/a^[k];
    END(*FOR*);
    y2[n-1]:= y2[n-1]/a^[n-1];
    y2[n-2]:= (y2[n-2]-y2[n-1]*c^[n-2])/a^[n-2];
    FOR k:= n-3 TO 0 BY -1 DO
      y2[k]:= (y2[k]-y2[k+1]*h^[k]-y2[n-1]*c^[k])/a^[k];
    END(*FOR*);
    y2[n]:= y2[0];
    FOR k:= 0 TO n-1 DO
      y1[k]:= (y[k+1]-y[k])/h^[k] -(y2[k+1]+2.0*y2[k])*h^[k]/6.0;
    END(*FOR*);
    y1[n]:= y1[0];
    Deallocate( h );
    Deallocate( a );
    Deallocate( c );
  END Spline2;


  PROCEDURE SplineCurve( n: INTEGER;
                         VAR x, y, t, x1, x2, y1, y2: ARRAY OF REAL );
    VAR  k: INTEGER;
  BEGIN
    t[0]:= 0.0;
    FOR k:= 1 TO n DO
      t[k]:= t[k-1] +Sqrt(Sqr(x[k]-x[k-1]) +Sqr(y[k]-y[k-1]));
    END(*FOR*);
    FOR k:= 1 TO n DO
      t[k]:= t[k]/t[n];
    END(*FOR*);
    Spline( n, t, x, x1, x2 );
    Spline( n, t, y, y1, y2 );
  END SplineCurve;


  PROCEDURE SplineCurve2( n: INTEGER;
                          VAR x, y, t, x1, x2, y1, y2: ARRAY OF REAL );
    VAR  k: INTEGER;
  BEGIN
    t[0]:= 0.0;
    FOR k:= 1 TO n DO
      t[k]:= t[k-1] +Sqrt(Sqr(x[k]-x[k-1]) +Sqr(y[k]-y[k-1]));
    END(*FOR*);
    FOR k:= 1 TO n DO
      t[k]:= t[k]/t[n];
    END(*FOR*);
    Spline2( n, t, x, x1, x2 );
    Spline2( n, t, y, y1, y2 );
  END SplineCurve2;


  PROCEDURE Bezier( x0, y0, x1, y1, x2, y2, x3, y3: REAL );
    VAR
      i, j, xl, yl, xn, yn: INTEGER;
      t, x, y,
      ax, bx, cx,
      ay, by, cy : REAL;
  BEGIN
    ax:= (x1-x0)*3.0;
    bx:= (x2-x1)*3.0 -ax;
    cx:= x3-x0-bx-ax;
    ay:= (y1-y0)*3.0;
    by:= (y2-y1)*3.0 -ay;
    cy:= y3-y0-by-ay;
    x:= ABS(bx +3.0*cx);
    IF (x < ABS(bx)) THEN  x:= ABS(bx);  END(*IF*);
    y:= ABS(by+ 3.0*cy);
    IF (y < ABS(by)) THEN  y:= ABS(by);  END(*IF*);
    IF (x < y) THEN  j:= TRUNC(Sqrt(y)+0.5);
    ELSE             j:= TRUNC(Sqrt(x)+0.5);
    END(*IF*);
    IF (j < 1) THEN j:= 1;  END(*IF*);
    SetUCPen( x0, y0 );
    FOR i:= 0 TO j DO
      t:= FLOAT(i)/FLOAT(j);
      x:= ((cx*t+bx)*t+ax)*t+x0;
      y:= ((cy*t+by)*t+ay)*t+y0;
      UCLineTo( x, y );
    END(*FOR*);
  END Bezier;


  PROCEDURE PlotSpline( n: INTEGER;
                        VAR x, y, y1: ARRAY OF REAL );
    VAR
      i: INTEGER;
      h: REAL;
  BEGIN
    FOR i:= 0 TO n-1 DO
      h:= (x[i+1]-x[i])/3.0;
      Bezier( x[i], y[i], x[i]+h, y[i]+y1[i]*h,
              x[i+1]-h, y[i+1]-y1[i+1]*h, x[i+1], y[i+1] );
    END(*FOR*);
  END PlotSpline;


  PROCEDURE PlotSpline2( n: INTEGER;
                        VAR t, x, x1, y, y1: ARRAY OF REAL );
    VAR
      i: INTEGER;
      h: REAL;
  BEGIN
    FOR i:= 0 TO n-1 DO
      h:= (t[i+1]-t[i])/3.0;
      Bezier( x[i], y[i], x[i]+x1[i]*h, y[i]+y1[i]*h,
              x[i+1]-x1[i+1]*h, y[i+1]-y1[i+1]*h, x[i+1], y[i+1] );
    END(*FOR*);
  END PlotSpline2;


END Splines.
