
 IMPLEMENTATION MODULE StandardIO;

  (******************************************************************)
  (*                                                                *)
  (*        MODULE: StandardIO            Version   0.1 / 001       *)
  (*                                                                *)
  (*        Purpose:                                                *)
  (*                                                                *)
  (*        Implementation:  For Apple Macintosh                    *)
  (*                                                                *)
  (*        Programming:     O. Roth                                *)
  (*                         Institut fr Phytomedizin              *)
  (*                         ETH-Zentrum /CLS                       *)
  (*                         CH-8092 Zrich                         *)
  (*                                                                *)
  (*        LAST MODIFICATION :   29/Mai/93 af                      *)
  (******************************************************************)

  (*

        Implementation and Revisions:
        ============================

        Author  Date        Description
        ------  ----        -----------

        af      10/03/89    MakeStandardIOWindow created so that it
        					fills the screen
        					NOTE (bug?): Autorestore is not functioning
        					since the DM is actually not running
        					
       af		29/05/93	Avoiding of hanging of a Read procedure
        					if not standard IO window actually used

  *)


 IMPORT ASCII;

	      
 FROM DMWindows IMPORT Window, notExistingWindow,
                       WindowKind, ScrollBars, CloseAttr, ZoomAttr,
                       WindowFrame, WFFixPoint, CreateWindow, PutOnTop,
                       WindowExists, AutoRestoreProc, AddWindowHandler,
                       WindowHandlers;
                              
  FROM DMWindowIO IMPORT SelectForOutput, SetPos, GetPos,
                         EraseContent,
                         AssignWindowFont, WindowFont,
                         BackgroundWidth, BackgroundHeight;
                         
  FROM DMSystem IMPORT TitleBarHeight;
  
  IMPORT DMWindowIO;

  FROM DMConversions IMPORT StringToReal, StringToInt, StringToCard;

 IMPORT DMMaster;


  (******************************************************************)


  (*. CONST
    left = 10; bottom = 10; width = 490; height = 280; .*)


  VAR
    w  : Window;
    wf : WindowFrame;
    oldKBHdlMode: BOOLEAN;
    oldPrio: INTEGER;
    dmReadingActive, wasForced: BOOLEAN;


  (******************************************************************)



  PROCEDURE SysBeep( millisec : INTEGER );
    PROCEDURE SystemBeep( duration : INTEGER ); CODE 0A9C8H;
  BEGIN
    SystemBeep( millisec );
  END SysBeep;


  PROCEDURE ActivateReading(u: Window);
  BEGIN
    DMMaster.GetKeyboardHandlerMode(oldKBHdlMode,oldPrio);
    dmReadingActive := TRUE;
    DMMaster.SetKeyboardHandlerMode(dmReadingActive(*readGetsThem*),0(*max priority*) );
  END ActivateReading;

  PROCEDURE DeactivateReading(u: Window);
  BEGIN
    DMMaster.SetKeyboardHandlerMode(oldKBHdlMode,oldPrio);
    dmReadingActive := FALSE;
  END DeactivateReading;

  PROCEDURE MakeStandardIOWindow;
    CONST m = 3;
  BEGIN

    IF NOT WindowExists(w) THEN
      wf.x:= m; wf.y:= m; 
      wf.w:= BackgroundWidth()-2*m; 
      wf.h:= BackgroundHeight()-TitleBarHeight() - 2*m;
      CreateWindow( w, GrowOrShrinkOrDrag, WithoutScrollBars,
                    WithCloseBox, WithoutZoomBox, bottomLeft, wf,
                    'Standard I/O', AutoRestoreProc );
      AddWindowHandler(w, broughtToFront, ActivateReading, 0(*high priority*) );
      AddWindowHandler(w, removedFromFront, DeactivateReading, 0(*high priority*) );
    END;
    PutOnTop( w );
    SelectForOutput( w );
    AssignWindowFont( Monaco, 9 );           
    EraseContent;
    SetPos( 1, 1 );

  END MakeStandardIOWindow;
  




  PROCEDURE WriteReal( x: REAL; field, dec : CARDINAL );
  BEGIN
    DMWindowIO.WriteReal( x, field, dec );
  END WriteReal;
  


  PROCEDURE WriteInt( i: INTEGER; field: CARDINAL );
  BEGIN
    DMWindowIO.WriteInt( i, field );
  END WriteInt;
  


  PROCEDURE WriteCard( K, field: CARDINAL );
  BEGIN
    DMWindowIO.WriteCard( K, field );
  END WriteCard;
  


  PROCEDURE Write( c: CHAR );
  BEGIN
    DMWindowIO.Write( c );
  END Write;
  


  PROCEDURE WriteString( str: ARRAY OF CHAR );
  BEGIN
    DMWindowIO.WriteString( str );
  END WriteString;



  PROCEDURE WriteLn;
  BEGIN
    DMWindowIO.WriteLn;
  END WriteLn;



  PROCEDURE CheckReadibility;  (* avoids hanging in new DMMaster, see DEF *)
  BEGIN
    wasForced := NOT dmReadingActive;
    IF NOT dmReadingActive THEN
      ActivateReading(notExistingWindow);
    END(*IF*);
  END CheckReadibility;
  
  PROCEDURE RestoreOldReadibility;
  BEGIN
    IF wasForced THEN
      DeactivateReading(notExistingWindow);
    END(*IF*);
  END RestoreOldReadibility;
  

  PROCEDURE Read( VAR c: CHAR );
  BEGIN
    CheckReadibility;
    DMMaster.Read( c );
    DMWindowIO.Write( c );
    RestoreOldReadibility;
  END Read;



  PROCEDURE ReadLn( VAR str: ARRAY OF CHAR );
    VAR
      i: INTEGER;
      lin0, col0 : CARDINAL;
      lin, col: CARDINAL;
      c: CHAR;
  BEGIN
    CheckReadibility;
    i:= 0;
    DMWindowIO.GetPos( lin0, col0 );
    REPEAT
      DMMaster.Read( c );
      IF (c < ' ') THEN
        IF ( c = ASCII.bs ) AND ( i > 0) THEN
          DEC( i );
          str[i]:= ASCII.nul;
          DMWindowIO.GetPos( lin, col   );
          DMWindowIO.SetPos( lin, col-2 );
          DMWindowIO.WriteString('   ');
          DMWindowIO.SetPos( lin0, col0 );
          DMWindowIO.WriteString(str);
        ELSE
          IF ( c # ASCII.cr ) THEN
            SysBeep( 5 );
          END;
        END; (* IF *)
      ELSE
        DMWindowIO.Write( c );
        str[i]:= c;
        INC( i );
      END; (* IF *)
    UNTIL ( c  = ASCII.cr );
    str[i]:= ASCII.nul;
    DMWindowIO.WriteLn;
    
    RestoreOldReadibility;    
  END ReadLn;



  PROCEDURE ReadReal( VAR x: REAL );

    VAR
      i, j, len     : INTEGER;
      c             : CHAR;
      rstr          : ARRAY [0..20] OF CHAR;
      LEGAL, OK, FINISH  : BOOLEAN;
      lin, col      : CARDINAL;
      lin0, col0    : CARDINAL;

  BEGIN
    CheckReadibility;

    i:= 0;
    DMWindowIO.GetPos( lin0, col0 );
    FINISH:= FALSE;

    REPEAT

      REPEAT
        DMMaster.Read( c );
        IF (c <= ' ') THEN
          LEGAL:= FALSE;
          IF ( c = ASCII.bs ) AND ( i > 0) THEN
            DEC( i );
            rstr[i]:= ASCII.nul;
            DMWindowIO.GetPos( lin, col   );
            DMWindowIO.SetPos( lin, col-2 );
            DMWindowIO.WriteString('   ');
            DMWindowIO.SetPos( lin0, col0 );
            DMWindowIO.WriteString( rstr );
          ELSE
            IF ( c = ASCII.bs ) (* BUT i = 0 *) THEN
              SysBeep( 1 );
            ELSE
              FINISH:= TRUE;
            END;
          END; (* IF *)
        ELSE
          LEGAL:= ( ( c >= '0') AND ( c <= '9' )
                  OR  ( c = '+' )
                  OR  ( c = '-' )
                  OR  ( c = '.' )
                  OR  ( CAP(c) = 'E' ) );
          IF NOT LEGAL THEN SysBeep( 5 ); END;
        END;
      UNTIL LEGAL OR FINISH;

      IF LEGAL THEN
        DMWindowIO.Write( c );
        rstr[i]:= c;
        INC( i );
      END; (* IF *)

    UNTIL FINISH;

    rstr[i]:= ASCII.nul;

    StringToReal( rstr, x, OK );
    
    IF NOT OK THEN
      SysBeep( 20 );
      len:= i + 3;
      DMWindowIO.SetPos( lin0, col0 );
      FOR j:=1 TO len DO
        DMWindowIO.Write(' ');
      END;
      DMWindowIO.SetPos( lin0, col0 );
      ReadReal( x );
    ELSE
      IF ( c = ASCII.cr ) THEN WriteLn;
      ELSE       DMWindowIO.Write(' ');   
      END;
    END; (* IF *)

    RestoreOldReadibility;
  END ReadReal;
  



  PROCEDURE ReadInt( VAR I: INTEGER);

    VAR
      ii, j, len     : INTEGER;
      c              : CHAR;
      rstr           : ARRAY [0..20] OF CHAR;
      LEGAL, OK, FINISH  : BOOLEAN;
      lin, col       : CARDINAL;
      lin0, col0     : CARDINAL;

  BEGIN
    CheckReadibility;

    ii:= 0;
    DMWindowIO.GetPos( lin0, col0 );
    FINISH:= FALSE;

    REPEAT

      REPEAT
        DMMaster.Read( c );
        IF (c <= ' ') THEN
          LEGAL:= FALSE;
          IF ( c = ASCII.bs ) AND ( ii > 0) THEN
            DEC( ii );
            rstr[ii]:= ASCII.nul;
            DMWindowIO.GetPos( lin, col   );
            DMWindowIO.SetPos( lin, col-2 );
            DMWindowIO.WriteString('   ');
            DMWindowIO.SetPos( lin0, col0 );
            DMWindowIO.WriteString( rstr );
          ELSE
            IF ( c = ASCII.bs ) (* BUT ii = 0 *) THEN
              SysBeep( 1 );
            ELSE
              FINISH:= TRUE;
            END;
          END; (* IF *)
        ELSE
          LEGAL:= ( ( c >= '0') AND ( c <= '9' )
                  OR  ( c = '+' )
                  OR  ( c = '-' )
                  OR  ( CAP(c) = 'E' ) );
          IF NOT LEGAL THEN SysBeep( 5 ); END;
        END;
      UNTIL LEGAL OR FINISH;

      IF LEGAL THEN
        DMWindowIO.Write( c );
        rstr[ii]:= c;
        INC( ii );
      END; (* IF *)

    UNTIL FINISH;

    rstr[ii]:= ASCII.nul;

    StringToInt( rstr, I, OK );
    
    IF NOT OK THEN
      SysBeep( 20 );
      len:= ii + 3;
      DMWindowIO.SetPos( lin0, col0 );
      FOR j:=1 TO len DO
        DMWindowIO.Write(' ');
      END;
      DMWindowIO.SetPos( lin0, col0 );
      ReadInt( I );
    ELSE
      IF ( c = ASCII.cr ) THEN WriteLn;
      ELSE       DMWindowIO.Write(' ');   
      END;
    END; (* IF *)

    RestoreOldReadibility;
  END ReadInt;


  PROCEDURE ReadCard( VAR k: CARDINAL );

    VAR
      ii, j, len     : INTEGER;
      c              : CHAR;
      str           : ARRAY [0..20] OF CHAR;
      LEGAL, OK, FINISH  : BOOLEAN;
      lin, col       : CARDINAL;
      lin0, col0     : CARDINAL;

  BEGIN
    CheckReadibility;

    ii:= 0;
    DMWindowIO.GetPos( lin0, col0 );
    FINISH:= FALSE;

    REPEAT

      REPEAT
        DMMaster.Read( c );
        IF (c <= ' ') THEN
          LEGAL:= FALSE;
          IF ( c = ASCII.bs ) AND ( ii > 0) THEN
            DEC( ii );
            str[ii]:= ASCII.nul;
            DMWindowIO.GetPos( lin, col   );
            DMWindowIO.SetPos( lin, col-2 );
            DMWindowIO.WriteString('   ');
            DMWindowIO.SetPos( lin0, col0 );
            DMWindowIO.WriteString( str );
          ELSE
            IF ( c = ASCII.bs ) (* BUT ii = 0 *) THEN
              SysBeep( 1 );
            ELSE
              FINISH:= TRUE;
            END;
          END; (* IF *)
        ELSE
          LEGAL:= ( ( c >= '0') AND ( c <= '9' )
                  OR  ( c = '+' )
                  OR  ( CAP(c) = 'E' ) );
          IF NOT LEGAL THEN SysBeep( 5 ); END;
        END;
      UNTIL LEGAL OR FINISH;

      IF LEGAL THEN
        DMWindowIO.Write( c );
        str[ii]:= c;
        INC( ii );
      END; (* IF *)

    UNTIL FINISH;

    str[ii]:= ASCII.nul;

    StringToCard( str, k, OK );
    
    IF NOT OK THEN
      SysBeep( 20 );
      len:= ii + 3;
      DMWindowIO.SetPos( lin0, col0 );
      FOR j:=1 TO len DO
        DMWindowIO.Write(' ');
      END;
      DMWindowIO.SetPos( lin0, col0 );
      ReadCard( k );
    ELSE
      IF ( c = ASCII.cr ) THEN WriteLn;
      ELSE       DMWindowIO.Write(' ');   
      END;
    END; (* IF *)

    RestoreOldReadibility;
  END ReadCard;


BEGIN
  oldKBHdlMode := FALSE;
  oldPrio:= 0;
  dmReadingActive := FALSE;
  wasForced := FALSE;
END StandardIO.
