IMPLEMENTATION MODULE WriteDatTim;

  (*

        Implementation and Revisions:
        ============================

        Author  Date        Description
        ------  ----        -----------

        af      16/05/88    First implementation (MacMETH 2.4)
        af		25/11/90	DateAndTimeRec now defined in
        					this module, no other modules are
        					imported any more.
  *)
  
  
  VAR 
    curWrite: WriteProc;
  
  PROCEDURE WriteString(s: ARRAY OF CHAR);
    VAR i,n: INTEGER;
  BEGIN
    i:= 0; n:= HIGH(s);
    WHILE (i<=n) AND (s[i]<>0C) DO curWrite(s[i]); INC(i) END;
  END WriteString;
  
  PROCEDURE WriteInt(x: INTEGER; n: CARDINAL);
    VAR i: CARDINAL; x0: CARDINAL;
      a: ARRAY [0..11] OF CHAR;
  BEGIN (*WriteInt*)
    i := 0; x0 := ABS(x);
    REPEAT a[i] := CHR(x0 MOD 10 + ORD("0"));
      x0 := x0 DIV 10; INC(i)
    UNTIL x0 = 0;
    IF x < 0 THEN a[i] := "-"; INC(i) END ;
    WHILE n > i DO
      DEC(n); curWrite("0")
    END ;
    REPEAT DEC(i); curWrite(a[i]) UNTIL i = 0
  END WriteInt;
  

  PROCEDURE WriteDate(d: DateAndTimeRec; w: WriteProc; df: DateFormat);
  BEGIN
    curWrite:= w;
    WITH d DO
    CASE df OF
      brief: (* e.g. 31/05/88 *)
        WriteInt(day,2); WriteString("/");
        WriteInt(month,2); WriteString("/");
        WriteInt(year MOD 100,0);
    | letMonth: (*e.g. 31/Mai/1988 *)
        WriteInt(day,2); WriteString("/");
        CASE month OF
          Jan   : WriteString("Jan");
        | Feb   : WriteString("Feb");
        | Mar   : WriteString("Mar");
        | Apr   : WriteString("Apr");
        | Mai   : WriteString("Mai");
        | Jun   : WriteString("Jun");
        | Jul   : WriteString("Jul");
        | Aug   : WriteString("Aug");
        | Sep   : WriteString("Sep");
        | Oct   : WriteString("Oct");
        | Nov   : WriteString("Nov");
        | Dec   : WriteString("Dec");
        END(*CASE*);
        WriteString("/"); WriteInt(year,0);
    | full: (* 31st Mai 1988 *)
        WriteInt(day,0); 
        IF day=1 THEN WriteString("st ")
        ELSIF day=2 THEN WriteString("nd ")
        ELSIF day=3 THEN WriteString("rd ")
        ELSE
          WriteString(" ");
        END;
        CASE month OF
          Jan   : WriteString("January");
        | Feb   : WriteString("February");
        | Mar   : WriteString("March");
        | Apr   : WriteString("April");
        | Mai   : WriteString("Mai");
        | Jun   : WriteString("June");
        | Jul   : WriteString("Juli");
        | Aug   : WriteString("August");
        | Sep   : WriteString("September");
        | Oct   : WriteString("October");
        | Nov   : WriteString("November");
        | Dec   : WriteString("December");
        END(*CASE*);
        WriteString(" "); WriteInt(year,0);
    END(*CASE*);
    END(*WITH*);
  END WriteDate;
  
  PROCEDURE WriteTime(d: DateAndTimeRec; w: WriteProc; tf: TimeFormat);
  BEGIN                       
    curWrite:= w;
    CASE tf OF
      brief24h: (* e.g. 23:15 *)
        WriteInt(d.hour,2); WriteString(":"); WriteInt(d.minute,2);
    | brief24hSecs: (* e.g. 23:15:02 *)
        WriteInt(d.hour,2); WriteString(":"); 
        WriteInt(d.minute,2); WriteString(":"); 
        WriteInt(d.second,2);
    | let24hSecs: (* e.g. 23h 15' 02" *)
        WriteInt(d.hour,2); WriteString("h "); 
        WriteInt(d.minute,2); WriteString("' ");
        WriteInt(d.second,2); WriteString('"');
    | full24hSecs: (* e.g. 23 hours 15 minutes 02 seconds *)
        WriteInt(d.hour,2); WriteString(" hours "); 
        WriteInt(d.minute,2); WriteString(" minutes ");
        WriteInt(d.second,2); WriteString(' seconds');
    | brief12h: (* e.g. 11:15pm *)
        WriteInt(d.hour MOD 12,2); WriteString(":"); 
        WriteInt(d.minute,2);
        IF d.hour>12 THEN WriteString("pm") ELSE WriteString("am") END;
    END(*CASE*);
  END WriteTime;


END WriteDatTim.
