
IMPLEMENTATION MODULE WriteRTF;

  (*******************************************************************

    Module  WriteRTF     (Version 0.1)

           Programming and Implementation
              O. Roth             15.04.89
              
          Swiss Federal Institute of Technology Zurich
          ETH-Zentrum
          CH-8092 Zurich
          Switzerland

          Last revision:   15.04.89    or

  *******************************************************************)


  FROM DMFiles    IMPORT
                  TextFile, Response,
                  CreateNewFile, Lookup, Close,
                  WriteChar, WriteChars, WriteEOL,
                  PutCardinal, PutInteger, PutReal, PutRealSci;

  FROM FileInfo IMPORT WriteFileStatus;

  FROM DMWindows  IMPORT Window;

  FROM DMConversions IMPORT IntToString;


  VAR
    styleMrk : ARRAY[plain..dflt]       OF ARRAY[0..7] OF CHAR;
    fontMrk  : ARRAY[Chicago..DfltFont] OF ARRAY[0..7] OF CHAR;
    sizeMrk  : ARRAY[0..7] OF CHAR;
    DfltStyle: RTFStyle;



  PROCEDURE WriteRTFHeader ( f: RTFFile );
  BEGIN
    WriteChars( f.textf, '{\rtf0\mac {\fonttbl{\f0\fswiss Chicago;}' );
    WriteChars( f.textf, '{\f2\froman New York;}{\f3\fswiss Geneva;}{\f4\fmodern Monaco;}' );
    WriteChars( f.textf, '{\f14\fnil Bookman;}{\f15\fnil N Helvetica Narrow;}{\f16\fnil Palatino;}' );
    WriteChars( f.textf, '{\f20\froman Times;}{\f21\fswiss Helvetica;}{\f22\fmodern Courier;}' );
    WriteEOL( f.textf );
    WriteChars( f.textf, '{\f23\ftech Symbol;}{\f33\fnil Avant Garde;}}' );
    WriteEOL( f.textf );
  END WriteRTFHeader;



  PROCEDURE SetDfltFontStyleSize( VAR f: RTFFile );
    VAR fSizeStr: ARRAY[0..2] OF CHAR;
  BEGIN
    f.curFont := DfltFont;
    f.curStyle:= DfltStyle;
    f.curSize := 0;
  END SetDfltFontStyleSize;



  PROCEDURE CreateRTFFile ( VAR f  : RTFFile;
                            prompt, defaultName: ARRAY OF CHAR );
  BEGIN
    CreateNewFile( f.textf, prompt, defaultName );
    IF (f.textf.res = done) THEN
      WriteRTFHeader( f );
      SetDfltFontStyleSize( f );
    ELSE
      WriteFileStatus( f.textf );
    END(*IF*);
  END CreateRTFFile;



  PROCEDURE RewriteRTFFile ( VAR f  : RTFFile;
                             fname  : ARRAY OF CHAR );
  BEGIN
    Lookup( f.textf, fname, TRUE );
    IF (f.textf.res = done) THEN
      WriteRTFHeader( f );
      SetDfltFontStyleSize( f );
    ELSE
      WriteFileStatus( f.textf );
    END(*IF*);
  END RewriteRTFFile;



  PROCEDURE CloseRTFFile (  f  : RTFFile );
  BEGIN
    IF (f.curStyle # DfltStyle) THEN
      WriteChar( f.textf, '}' );
    END(*IF*);
    WriteRTFEOP( f );
    WriteChar(   f.textf, '}' );
    Close(       f.textf   );
  END CloseRTFFile;





  PROCEDURE WriteRTFChar (  f  : RTFFile;  ch: CHAR );
  BEGIN
    WriteChar( f.textf, ch );
  END WriteRTFChar;


  PROCEDURE WriteRTFChars ( f  : RTFFile;  str: ARRAY OF CHAR );
  BEGIN
    WriteChars( f.textf, str );
  END WriteRTFChars;


  PROCEDURE WriteRTFEOL (   f  : RTFFile ); (* End Of Line *)
  BEGIN
    WriteChars( f.textf, '\line' );
    WriteEOL(   f.textf );
  END WriteRTFEOL;


  PROCEDURE WriteRTFEOP (   f  : RTFFile ); (* End Of Paragraph *)
  BEGIN
    WriteChars( f.textf, '\par' );
    WriteEOL(   f.textf );
  END WriteRTFEOP;


  PROCEDURE WriteRTFTab (   f  : RTFFile ); (* Tabulator *)
  BEGIN
    WriteChars( f.textf, '\tab' );
  END WriteRTFTab;


  PROCEDURE WriteRTFPageBrk ( f  : RTFFile ); (* Page Break *)
  BEGIN
    WriteChars( f.textf, '\page' );
    WriteEOL(   f.textf );
  END WriteRTFPageBrk;


  PROCEDURE WriteRTFSectMrk ( f  : RTFFile );  (* Section Mark *)
  BEGIN
    WriteChars( f.textf, '\sect' );
    WriteEOL(   f.textf );
  END WriteRTFSectMrk;




  PROCEDURE PutRTFCardinal ( f  : RTFFile;  card: CARDINAL;  n: CARDINAL );
  BEGIN
    PutCardinal( f.textf, card, 1 );
  END PutRTFCardinal;


  PROCEDURE PutRTFInteger (  f  : RTFFile;  int:  INTEGER;   n: CARDINAL );
  BEGIN
    PutInteger( f.textf, int, 1 );
  END PutRTFInteger;


  PROCEDURE PutRTFReal (     f  : RTFFile;  real: REAL;   n, d: CARDINAL );
  BEGIN
    PutReal( f.textf, real, n, d );
  END PutRTFReal;


  PROCEDURE PutRTFRealSci (  f  : RTFFile;  real: REAL;      n: CARDINAL );
  BEGIN
    PutRealSci( f.textf, real, n );
  END PutRTFRealSci;





  PROCEDURE SetRTFFont (  VAR f: RTFFile;
                          Font : RTFFonts;
                          Size : CARDINAL;
                          Style: RTFStyle   );
    VAR
      NoChangesSoLong : BOOLEAN;
      styl            : RTFStyles;

  BEGIN

    NoChangesSoLong:= TRUE;

    IF (Style # f.curStyle) THEN
      IF (f.curStyle # DfltStyle) THEN WriteChar( f.textf, '}' ); END(*IF*);
      IF (Style      # DfltStyle) THEN WriteChar( f.textf, '{' ); END(*IF*);
      FOR styl:= bold TO hidden DO
        IF styl IN Style THEN
          WriteChars( f.textf, styleMrk[styl] );
        END(*IF*);
      END(*FOR*);
      NoChangesSoLong:= FALSE;
      f.curStyle:= Style;
    END(*IF*);

    IF (Size # f.curSize) THEN
      IF NoChangesSoLong THEN
        IF (f.curSize # 0) THEN  WriteChar( f.textf, '}' );  END(*IF*);
        IF (Size      # 0) THEN  WriteChar( f.textf, '{' );  END(*IF*);
      END(*IF*);
      IF (Size > 0) THEN
        WriteChars(  f.textf, sizeMrk );
        PutCardinal( f.textf, Size*2, 1 );
      END(*IF*);
      NoChangesSoLong:= FALSE;
      f.curSize:= Size;
    END(*IF*);

    IF (Font # f.curFont) THEN
      IF NoChangesSoLong THEN
        IF (f.curFont # DfltFont) THEN WriteChar( f.textf, '}' ); END(*IF*);
        IF (Font      # DfltFont) THEN WriteChar( f.textf, '{' ); END(*IF*);
      END(*IF*);
      WriteChars( f.textf, fontMrk[Font] );
      NoChangesSoLong:= FALSE;
      f.curFont:= Font;
    END(*IF*);

    IF NOT NoChangesSoLong THEN
      WriteChar( f.textf, ' ' );
    END(*IF*);

  END SetRTFFont;


  PROCEDURE GetRTFFont (  f        : RTFFile;
                          VAR Font : RTFFonts;
                          VAR Size : CARDINAL;
                          VAR Style: RTFStyle   );
  BEGIN
    Font:=  f.curFont;
    Size:=  f.curSize;
    Style:= f.curStyle;
  END GetRTFFont;



  PROCEDURE PutRTFPict ( f : RTFFile; fromWindow : Window );
  BEGIN
  END PutRTFPict;



BEGIN


  styleMrk[plain ]:= '\plain';
  styleMrk[bold  ]:= '\b';
  styleMrk[italic]:= '\i';
  styleMrk[underl]:= '\ul';
  styleMrk[outlin]:= '\outl';
  styleMrk[shadow]:= '\shad';
  styleMrk[subscr]:= '\dn4';
  styleMrk[supscr]:= '\up4';
  styleMrk[smcaps]:= '\scaps';
  styleMrk[alcaps]:= '\caps';
  styleMrk[hidden]:= '\v';
  styleMrk[dflt  ]:= '';

  sizeMrk:= '\fs';

  fontMrk[Chicago  ]:= '\f0';
  fontMrk[NewYork  ]:= '\f2';
  fontMrk[Geneva   ]:= '\f3';
  fontMrk[Monaco   ]:= '\f4';
  fontMrk[Bookman  ]:= '\f14';
  fontMrk[HelveticN]:= '\f15';
  fontMrk[Palatino ]:= '\f16';
  fontMrk[Times    ]:= '\f20';
  fontMrk[Helvetica]:= '\f21';
  fontMrk[Courier  ]:= '\f22';
  fontMrk[Symbol   ]:= '\f23';
  fontMrk[AvanGarde]:= '\f33';
  fontMrk[DfltFont ]:= '';

  DfltStyle:= RTFStyle{ dflt };

END WriteRTF.